@@ -144,8 +144,8 @@ static struct snd_cmi8330 {
 	struct snd_cmi8330_sb sb;
 	unsigned char image[10];
 #ifdef CONFIG_ISAPNP
-	struct isapnp_logdev *logcap;
-	struct isapnp_logdev *logplay;
+	struct isapnp_dev *cap;
+	struct isapnp_dev *play;
 #endif
 } *snd_cmi8330_cards[SND_CARDS] = SND_DEFAULT_PTR;
 
@@ -268,7 +268,7 @@ static int snd_cmi8330_resources(int dev, struct snd_cmi8330 *acard,
 	static int possible_dmas[] = {-1};
 	int err;
 
-	err = snd_register_ioport(card, snd_wssport[dev] + 4, 4,
+	err = snd_register_ioport(card, snd_wssport[dev],  8,
 				  "CMI8330/C3D (AD1848)", NULL);
 	if (err < 0) {
 		snd_printk("CMI8330/C3D (AD1848) could not register ioports\n");
@@ -438,97 +438,89 @@ static int snd_cmi8330_mixer(struct snd_cmi8330 *acard, snd_card_t * card)
 #ifdef CONFIG_ISAPNP
 static int snd_cmi8330_isapnp(int dev, struct snd_cmi8330 *acard)
 {
+	static int idx = 0;
+	static struct isapnp_card *card = NULL;
 	struct isapnp_dev *pdev;
-	struct isapnp_config cfg;
-	int i;
 	unsigned int tmp;
 
-	pdev = NULL;
-
-	for (i = 0; (tmp = snd_cmi8330_pnpids[i]); i += 3)
-		if ((pdev = isapnp_find_device(tmp >> 16, tmp & 0xffff, dev)))
+      __again:
+	acard->cap = acard->play = NULL;
+	while ((tmp = snd_cmi8330_pnpids[idx]) != 0) {
+		if ((card = isapnp_find_card(tmp >> 16, tmp & 0xffff, card)))
 			break;
+		idx += 3;
+	}
 
-	if (!pdev) {
+	if (!card) {
 		snd_printk("CMI8330/C3D PnP no device\n");
 		return -ENODEV;
 	}
-	tmp = snd_cmi8330_pnpids[i + 1];	/* capture */
-	acard->logcap = isapnp_find_logdev(pdev, tmp >> 16, tmp & 0xffff, 0);
+	tmp = snd_cmi8330_pnpids[idx + 1];	/* capture */
+	acard->cap = isapnp_find_dev(card, tmp >> 16, tmp & 0xffff, NULL);
 
-	if (!acard->logcap) {
-		snd_printk("CMI8330/C3D PnP no logical device\n");
-		return -ENXIO;
+	if (!acard->cap) {
+		snd_printdd("CMI8330/C3D PnP no logical device\n");
+		goto __again;
 	}
-	tmp = snd_cmi8330_pnpids[i + 2];	/* playback */
-	acard->logplay = isapnp_find_logdev(pdev, tmp >> 16, tmp & 0xffff, 0);
+	tmp = snd_cmi8330_pnpids[idx + 2];	/* playback */
+	acard->play = isapnp_find_dev(card, tmp >> 16, tmp & 0xffff, NULL);
 
-	if (isapnp_cfg_begin(acard->logcap->dev->csn, acard->logcap->number))
+	pdev = acard->cap;
+	if (pdev->prepare(pdev)<0)
 		return -EAGAIN;
-	if (isapnp_config_init(&cfg, acard->logcap)) {
-		isapnp_cfg_end();
-		return -EAGAIN;
-	}
 	/* allocate AD1848 resources */
 	if (snd_wssport[dev] != SND_AUTO_PORT)
-		cfg.port[0] = snd_wssport[dev];
+		isapnp_resource_change(&pdev->resource[0], snd_wssport[dev], 8);
 	if (snd_wssdma[dev] != SND_AUTO_DMA)
-		cfg.dma[0] = snd_wssdma[dev];
+		isapnp_resource_change(&pdev->dma_resource[0], snd_wssdma[dev], 1);
 	if (snd_wssirq[dev] != SND_AUTO_IRQ)
-		cfg.irq[0] = snd_wssirq[dev];
+		isapnp_resource_change(&pdev->irq_resource[0], snd_wssirq[dev], 1);
 
-	if (isapnp_configure(&cfg)) {
+	if (pdev->activate(pdev)<0) {
 		snd_printk("CMI8330/C3D (AD1848) PnP configure failure\n");
-		isapnp_cfg_end();
 		return -EBUSY;
 	}
-	snd_wssport[dev] = cfg.port[0];
-	snd_wssdma[dev] = cfg.dma[0];
-	snd_wssirq[dev] = cfg.irq[0];
-	isapnp_activate(acard->logcap->number);
+	snd_wssport[dev] = pdev->resource[0].start;
+	snd_wssdma[dev] = pdev->dma_resource[0].start;
+	snd_wssirq[dev] = pdev->irq_resource[0].start;
 
-	if (isapnp_config_init(&cfg, acard->logplay)) {
-		isapnp_cfg_end();
+	if (pdev->activate(pdev)<0)
+		return -EAGAIN;
+
+	/* allocate SB16 resources */
+	pdev = acard->play;
+	if (pdev->prepare(pdev)<0) {
+		acard->cap->deactivate(acard->cap);
 		return -EAGAIN;
 	}
-	/* allocate SB16 resources */
 	if (snd_sbport[dev] != SND_AUTO_PORT)
-		cfg.port[0] = snd_sbport[dev];
+		isapnp_resource_change(&pdev->resource[0], snd_sbport[dev], 16);
 	if (snd_sbdma8[dev] != SND_AUTO_DMA)
-		cfg.dma[0] = snd_sbdma8[dev];
+		isapnp_resource_change(&pdev->dma_resource[0], snd_sbdma8[dev], 1);
 	if (snd_sbdma16[dev] != SND_AUTO_DMA)
-		cfg.dma[1] = snd_sbdma16[dev];
+		isapnp_resource_change(&pdev->dma_resource[1], snd_sbdma16[dev], 1);
 	if (snd_sbirq[dev] != SND_AUTO_IRQ)
-		cfg.irq[0] = snd_sbirq[dev];
+		isapnp_resource_change(&pdev->irq_resource[0], snd_sbirq[dev], 1);
 
-	if (isapnp_configure(&cfg)) {
+	if (pdev->activate(pdev)<0) {
 		snd_printk("CMI8330/C3D (SB16) PnP configure failure\n");
-		isapnp_cfg_end();
+		acard->cap->deactivate(acard->cap);
 		return -EBUSY;
 	}
-	snd_sbport[dev] = cfg.port[0];
-	snd_sbdma8[dev] = cfg.dma[0];
-	snd_sbdma16[dev] = cfg.dma[1];
-	snd_sbirq[dev] = cfg.irq[0];
-	isapnp_activate(acard->logplay->number);
+	snd_sbport[dev] = pdev->resource[0].start;
+	snd_sbdma8[dev] = pdev->dma_resource[0].start;
+	snd_sbdma16[dev] = pdev->dma_resource[1].start;
+	snd_sbirq[dev] = pdev->irq_resource[0].start;
 
-	isapnp_cfg_end();
 	return 0;
 }
 
 static void snd_cmi8330_deactivate(struct snd_cmi8330 *acard)
 {
-	if (!acard->logcap)
-		return;
-
-	if (isapnp_cfg_begin(acard->logcap->dev->csn, acard->logcap->number) < 0)
-		return;
-	isapnp_deactivate(acard->logcap->number);
-
-	if (acard->logplay)
-		isapnp_deactivate(acard->logplay->number);
-
-	isapnp_cfg_end();
+	if (acard->cap)
+		acard->cap->deactivate(acard->cap);
+	if (acard->play)
+		acard->play->deactivate(acard->play);
 }
 #endif
 
