@@ -82,8 +82,8 @@ static struct snd_audiodrive {
 	snd_hwdep_t *synth;
 	es18xx_t *codec;
 #ifdef CONFIG_ISAPNP
-	struct isapnp_logdev *logdev;
-	struct isapnp_logdev *logdevc;
+	struct isapnp_dev *dev;
+	struct isapnp_dev *devc;
 #endif
 } *snd_audiodrive_cards[SND_CARDS] = SND_DEFAULT_PTR;
 
@@ -274,109 +274,91 @@ static int snd_audiodrive_resources(int dev,
 #ifdef CONFIG_ISAPNP
 static int snd_audiodrive_isapnp(int dev, struct snd_audiodrive *acard)
 {
-        int idx, step;
+	static int idx = 0;
+	static struct isapnp_card *card = NULL;
+        int step = 3;
         unsigned int tmp;
-        unsigned short cport;
 	struct isapnp_dev *pdev;
-	struct isapnp_logdev *logdev;
-	struct isapnp_config cfg;
 
-	step = 3;
-	idx = 0;
       __again:
-      	pdev = NULL;
+      	acard->dev = acard->devc = NULL;
 	while ((tmp = snd_audiodrive_pnpids[idx]) != 0) {
-		pdev = isapnp_find_device(tmp >> 16, tmp & 0xffff, dev);
-		if (pdev)
+		card = isapnp_find_card(tmp >> 16, tmp & 0xffff, card);
+		if (card)
 			break;
 		idx += step;
 	}
-	if (!pdev) {
+	if (!card) {
 		snd_printdd("isapnp failed for ES18xx\n");
 		return -ENODEV;
 	}
 	tmp = snd_audiodrive_pnpids[idx+1];
-	acard->logdev = logdev = isapnp_find_logdev(pdev, tmp >> 16, tmp & 0xffff, 0);
-	if (!acard->logdev) {
+	acard->dev = pdev = isapnp_find_dev(card, tmp >> 16, tmp & 0xffff, NULL);
+	if (!pdev) {
                 snd_printdd("isapnp failed for ES18xx - %i\n", idx);
-                idx += step;
                 goto __again;           /* maybe we have another config */
         }
 	tmp = snd_audiodrive_pnpids[idx+2];
-	acard->logdevc = isapnp_find_logdev(pdev, tmp >> 16, tmp & 0xffff, 0);
-	if (!acard->logdevc) {
+	acard->devc = isapnp_find_dev(card, tmp >> 16, tmp & 0xffff, NULL);
+	if (!acard->devc) {
                 snd_printdd("isapnp failed for ES18xx (Control) - %i\n", idx);
-                idx += step;
                 goto __again;           /* maybe we have another config */
         }
-	if (isapnp_cfg_begin(logdev->dev->csn, acard->logdevc->number))
-		return -EAGAIN;
 	/* Control port initialization */
-	if (isapnp_config_init(&cfg, acard->logdevc)<0) {
-		isapnp_cfg_end();
+	if (acard->devc->prepare(acard->devc)<0)
 		return -EAGAIN;
-	}
-	if (isapnp_configure(&cfg) < 0) {
+	if (acard->devc->activate(acard->devc)<0) {
 		snd_printk("ES18xx isapnp control configure failure (out of resources?)\n");
-		isapnp_cfg_end();
 		return -EAGAIN;
 	}
-	cport = cfg.port[0];
 	snd_printdd("isapnp ES18xx: port=0x%x\n", cfg.port[0]);
 	/* PnP initialization */
-	if (isapnp_config_init(&cfg, logdev)) {
-		isapnp_cfg_end();
+	if (pdev->prepare(pdev)<0) {
+		acard->devc->deactivate(acard->devc);
 		return -EAGAIN;
 	}
 	if (snd_port[dev] != SND_AUTO_PORT)
-		cfg.port[0] = snd_port[dev];
+		isapnp_resource_change(&pdev->resource[0], snd_port[dev], 16);
 	if (snd_fm_port[dev] != SND_AUTO_PORT)
-		cfg.port[1] = snd_fm_port[dev];
+		isapnp_resource_change(&pdev->resource[1], snd_fm_port[dev], 4);
 	if (snd_mpu_port[dev] != SND_AUTO_PORT)
-		cfg.port[2] = snd_mpu_port[dev];
+		isapnp_resource_change(&pdev->resource[2], snd_mpu_port[dev], 2);
 	if (snd_dma1[dev] != SND_AUTO_DMA)
-		cfg.dma[0] = snd_dma1[dev];
+		isapnp_resource_change(&pdev->dma_resource[0], snd_dma1[dev], 1);
 	if (snd_dma2[dev] != SND_AUTO_DMA)
-		cfg.dma[1] = snd_dma2[dev];
+		isapnp_resource_change(&pdev->dma_resource[1], snd_dma2[dev], 1);
 	if (snd_irq[dev] != SND_AUTO_IRQ)
-		cfg.irq[0] = snd_irq[dev];
-	cfg.port_disable[0] = cport;
-	cfg.port_disable_size[0] = 8;
-	if (isapnp_configure(&cfg)<0) {
+		isapnp_resource_change(&pdev->irq_resource[0], snd_irq[dev], 1);
+	if (pdev->activate(pdev)<0) {
 		snd_printk("ES18xx isapnp configure failure (out of resources?)\n");
-		isapnp_cfg_end();
+		acard->devc->deactivate(acard->devc);
 		return -EBUSY;
 	}
 	/* ok. hack using Vendor-Defined Card-Level registers */
 	/* skip csn and logdev initialization - already done in isapnp_configure */
-	isapnp_cfg_set_byte(0x27, cfg.irq[0]);	/* Hardware Volume IRQ Number */
+	isapnp_cfg_begin(pdev->bus->number, pdev->devfn);
+	isapnp_write_byte(0x27, pdev->irq_resource[0].start);	/* Hardware Volume IRQ Number */
 	if (snd_mpu_port[dev] > SND_AUTO_PORT)
-		isapnp_cfg_set_byte(0x28, cfg.irq[0]);	/* MPU-401 IRQ Number */
-	isapnp_cfg_set_byte(0x72, cfg.irq[0]);	/* second IRQ */
-	snd_printdd("isapnp ES18xx: port=0x%x, fm port=0x%x, mpu port=0x%x\n", cfg.port[0], cfg.port[1], cfg.port[2]);
-	snd_printdd("isapnp ES18xx: dma1=%i, dma2=%i, irq=%i\n", cfg.dma[0], cfg.dma[1], cfg.irq[0]);
-	snd_port[dev] = cfg.port[0];
-	snd_fm_port[dev] = cfg.port[1];
-	snd_mpu_port[dev] = cfg.port[2];
-	snd_dma1[dev] = cfg.dma[0];
-	snd_dma2[dev] = cfg.dma[1];
-	snd_irq[dev] = cfg.irq[0];
-        isapnp_activate(acard->logdevc->number);
-        isapnp_activate(logdev->number);
+		isapnp_write_byte(0x28, pdev->irq);		/* MPU-401 IRQ Number */
+	isapnp_write_byte(0x72, pdev->irq_resource[0].start);	/* second IRQ */
 	isapnp_cfg_end();
+	snd_port[dev] = pdev->resource[0].start;
+	snd_fm_port[dev] = pdev->resource[1].start;
+	snd_mpu_port[dev] = pdev->resource[2].start;
+	snd_dma1[dev] = pdev->dma_resource[0].start;
+	snd_dma2[dev] = pdev->dma_resource[1].start;
+	snd_irq[dev] = pdev->irq_resource[0].start;
+	snd_printdd("isapnp ES18xx: port=0x%x, fm port=0x%x, mpu port=0x%x\n", snd_port[dev], snd_fm_port[dev], snd_mpu_port[dev]);
+	snd_printdd("isapnp ES18xx: dma1=%i, dma2=%i, irq=%i\n", snd_dma1[dev], snd_dma2[dev], snd_irq[dev]);
 	return 0;
 }
 
 static void snd_audiodrive_deactivate(struct snd_audiodrive *acard)
 {
-	if (!acard->logdevc)
-		return;
-	if (isapnp_cfg_begin(acard->logdevc->dev->csn, acard->logdevc->number)<0)
-		return;
-	isapnp_deactivate(acard->logdevc->number);
-	if (acard->logdev)
-		isapnp_deactivate(acard->logdev->number);
-	isapnp_cfg_end();
+	if (acard->devc)
+		acard->devc->deactivate(acard->devc);
+	if (acard->dev)
+		acard->dev->deactivate(acard->dev);
 }
 #endif /* CONFIG_ISAPNP */
 
