@@ -435,8 +435,10 @@ unsigned int cd_port_sel = 0xff;
 unsigned char game_sel = 0xff;
 
 #ifdef CONFIG_ISAPNP
-int opti9xx_isapnp_id = -1;
-struct isapnp_dev *opti9xx_isapnp_dev = NULL;
+static int opti9xx_isapnp_id = -1;
+static struct isapnp_card *opti9xx_isapnp_card = NULL;
+static struct isapnp_dev *opti9xx_isapnp_dev = NULL;
+static struct isapnp_dev *opti9xx_isapnp_dev_mpu = NULL;
 
 static unsigned int snd_opti9xx_isapnp_table[] = {
 	/* c924 */
@@ -1013,101 +1015,88 @@ int snd_opti9xx_config_pnp(void)
 {
 	int error;
 	unsigned int tmp;
-	struct isapnp_logdev *logdev;
-	struct isapnp_config cfg;
+	struct isapnp_dev *pdev;
 
 	/* normal configuration needed */
 	snd_opti9xx_config_isa();
 
 	/* audio device */
 	tmp = snd_opti9xx_isapnp_table[opti9xx_isapnp_id + 1];
-	logdev = isapnp_find_logdev(opti9xx_isapnp_dev, tmp >> 16,
-		tmp & 0xffff, 0); 
-	if (!logdev)
+	opti9xx_isapnp_dev = pdev = isapnp_find_dev(opti9xx_isapnp_card,
+						    tmp >> 16, tmp & 0xffff,
+						    NULL); 
+	if (!pdev)
 		return -EAGAIN;
+	if (pdev->activate)
+		return -EBUSY;
 
-	if ((error = isapnp_cfg_begin(logdev->dev->csn, logdev->number)))
-		return error;
-
-	if ((error = isapnp_config_init(&cfg, logdev))) {
-		isapnp_cfg_end();
-		return error;
-	}
+	if (pdev->prepare(pdev)<0)
+		return -EAGAIN;
 
 #define	PORT_OFFSET	(opti9xx_chip == c924) ? 1 : 0
 	if (snd_port != SND_AUTO_PORT)
-		cfg.port[0 + PORT_OFFSET] = WSS_BASE_PORT;
+		isapnp_resource_change(&pdev->resource[0], WSS_BASE_PORT, 8);
 	if (snd_irq != SND_AUTO_IRQ)
-		cfg.irq[0] = snd_irq;
+		isapnp_resource_change(&pdev->irq_resource[0], snd_irq, 1);
 	if (snd_dma1 != SND_AUTO_DMA)
-		cfg.dma[0] = snd_dma1;
+		isapnp_resource_change(&pdev->dma_resource[0], snd_dma1, 1);
 	if (snd_dma2 != SND_AUTO_DMA)
-		cfg.dma[1] = snd_dma2;
+		isapnp_resource_change(&pdev->dma_resource[1], snd_dma2, 1);
 	if (snd_opl_port != SND_AUTO_PORT)
-		cfg.port[1 + PORT_OFFSET] = snd_opl_port;
+		isapnp_resource_change(&pdev->resource[1 + PORT_OFFSET], snd_opl_port, 4);
 
-	if ((error = isapnp_configure(&cfg))) {
-		isapnp_cfg_end();
+	if ((error = pdev->activate(pdev))<0) {
 		return error;
 	}
 
-	snd_port = cfg.port[0 + PORT_OFFSET] - WSS_CONFIG_SIZE;
-	snd_irq = cfg.irq[0];
-	snd_dma1 = cfg.dma[0];
-	snd_dma2 = cfg.dma[1];
-	snd_opl_port = cfg.port[1 + PORT_OFFSET];
+	snd_port = pdev->resource[0 + PORT_OFFSET].start - WSS_CONFIG_SIZE;
+	snd_irq = pdev->irq_resource[0].start;
+	snd_dma1 = pdev->dma_resource[0].start;
+	snd_dma2 = pdev->dma_resource[1].start;
+	snd_opl_port = pdev->resource[1 + PORT_OFFSET].start;
 	if (opti9xx_chip > c930)
-		OPTi93X_MC_INDEX_IREG = cfg.port[3] + 2;
-
-	isapnp_activate(logdev->number);
-	isapnp_cfg_end();
+		OPTi93X_MC_INDEX_IREG = pdev->resource[3].start + 2;
 
 	/* game port */
 	tmp = snd_opti9xx_isapnp_table[opti9xx_isapnp_id + 2];
-	logdev = isapnp_find_logdev(opti9xx_isapnp_dev, tmp >> 16,
-		tmp & 0xffff, 0); 
-	if (logdev) {
-		if ((error = isapnp_cfg_begin(logdev->dev->csn, logdev->number)))
-			return error;
-
-		if (!game_sel)
-			isapnp_activate(logdev->number);
-		else
-			isapnp_deactivate(logdev->number);
-
-		isapnp_cfg_end();
+	pdev = isapnp_find_dev(opti9xx_isapnp_card, tmp >> 16,
+			       tmp & 0xffff, NULL); 
+	if (pdev) {
+		if (pdev->prepare(pdev)>=0) {
+			pdev->activate(pdev);
+			if (game_sel)
+				pdev->deactivate(pdev);
+		}
 	}
 
 	/* mpu-401 */
 	tmp = snd_opti9xx_isapnp_table[opti9xx_isapnp_id + 3];
-	logdev = isapnp_find_logdev(opti9xx_isapnp_dev, tmp >> 16,
-		tmp & 0xffff, 0); 
-	if (!logdev)
+	pdev = isapnp_find_dev(opti9xx_isapnp_card, tmp >> 16,
+			       tmp & 0xffff, NULL); 
+	opti9xx_isapnp_dev_mpu = pdev;
+	if (!pdev) {
+		opti9xx_isapnp_dev->deactivate(opti9xx_isapnp_dev);
 		return -EAGAIN;
+	}
 
-	if ((error = isapnp_cfg_begin(logdev->dev->csn, logdev->number)))
-		return error;
-
-	if ((error = isapnp_config_init(&cfg, logdev))) {
-		isapnp_cfg_end();
+	if (pdev->prepare(pdev)<0) {
+		opti9xx_isapnp_dev->deactivate(opti9xx_isapnp_dev);
 		return error;
 	}
 
 	if (snd_mpu_port != SND_AUTO_PORT)
-		cfg.port[0] = snd_mpu_port;
+		isapnp_resource_change(&pdev->resource[0], snd_mpu_port, 2);
 	if (snd_mpu_irq != SND_AUTO_IRQ)
-		cfg.irq[0] = snd_mpu_irq;
+		isapnp_resource_change(&pdev->irq_resource[0], snd_mpu_irq, 1);
 
-	if ((error = isapnp_configure(&cfg))) {
-		isapnp_cfg_end();
+	if ((error = pdev->activate(pdev))<0) {
+		opti9xx_isapnp_dev->deactivate(opti9xx_isapnp_dev);
 		return error;
 	}
 
-	snd_mpu_port = cfg.port[0];
-	snd_mpu_irq = cfg.irq[0];
+	snd_mpu_port = pdev->resource[0].start;
+	snd_mpu_irq = pdev->irq_resource[0].start;
 
-	isapnp_activate(logdev->number);
-	isapnp_cfg_end();
 	return ESUCCESS;
 }
 #endif	/* CONFIG_ISAPNP */
@@ -1223,8 +1212,8 @@ int snd_opti9xx_detect_pnp(void)
 	for (i = 0; (opti9xx_chip = tmp = snd_opti9xx_isapnp_table[i++]);) {
 		tmp = snd_opti9xx_isapnp_table[i++];
 
-		if ((opti9xx_isapnp_dev = isapnp_find_device(tmp >> 16,
-			tmp & 0xffff, 0))) {
+		if ((opti9xx_isapnp_card = isapnp_find_card(tmp >> 16,
+			tmp & 0xffff, NULL))) {
 			opti9xx_isapnp_id = i - 1;
 #ifdef OPTi9XX_DEBUG
 			opti9xx_printk("found an OPTi %s chip (PnP)\n",
@@ -1328,6 +1317,12 @@ void cleanup_module(void)
 		snd_mixer_unregister(snd_mixer);
 	if (snd_pcm)
 		snd_pcm_unregister(snd_pcm);
+#ifdef CONFIG_ISAPNP
+	if (opti9xx_isapnp_dev)
+		opti9xx_isapnp_dev->deactivate(opti9xx_isapnp_dev);
+	if (opti9xx_isapnp_dev_mpu)
+		opti9xx_isapnp_dev_mpu->deactivate(opti9xx_isapnp_dev_mpu);
+#endif
 	if (opti9xx_card)
 		snd_card_free(opti9xx_card);
 }
