@@ -97,19 +97,15 @@ static unsigned int snd_wavefront_pnpids[] = {
 static int 
 snd_wavefront_isapnp (int dev, snd_wavefront_card_t *acard, int pnp_id)
 {
-	int idx;
+	static struct isapnp_card *card = NULL;
 	unsigned int tmp;
 	struct isapnp_dev *pdev;
-	struct isapnp_config cfg;
     
-	idx = 0;
-
-	pdev = NULL;
 	tmp = (ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(pnp_id);
 
 	/* Find the device */
 
-	if ((pdev = isapnp_find_device (tmp >> 16, tmp & 0xffff, dev)) == 0) {
+	if ((card = isapnp_find_card (tmp >> 16, tmp & 0xffff, card)) == NULL) {
 		snd_printk ("isapnp failed for device\n");
 		return -ENODEV;
 	}
@@ -119,7 +115,7 @@ snd_wavefront_isapnp (int dev, snd_wavefront_card_t *acard, int pnp_id)
 	/* "windows sound system" is logical device 1 */
 
 	tmp = snd_wavefront_pnpids[0];
-	acard->wss = isapnp_find_logdev(pdev, tmp >> 16, tmp & 0xffff, 0);
+	acard->wss = isapnp_find_dev(card, tmp >> 16, tmp & 0xffff, NULL);
 	if (!acard->wss) {
 		snd_printk (LOGNAME "isapnp failed for WSS device\n");
 		return -ENODEV;
@@ -128,7 +124,7 @@ snd_wavefront_isapnp (int dev, snd_wavefront_card_t *acard, int pnp_id)
 	/* control interface is logical device 2 */
 
 	tmp = snd_wavefront_pnpids[1];
-	acard->ctrl = isapnp_find_logdev (pdev, tmp >> 16, tmp & 0xffff, 0);
+	acard->ctrl = isapnp_find_dev(card, tmp >> 16, tmp & 0xffff, NULL);
 	if (!acard->ctrl) {
 		snd_printk (LOGNAME "isapnp failed for CTRL device\n");
 		return -ENODEV;
@@ -137,7 +133,7 @@ snd_wavefront_isapnp (int dev, snd_wavefront_card_t *acard, int pnp_id)
 	/* synth is logical device 4 */
 
 	tmp = snd_wavefront_pnpids[3];
-	acard->synth = isapnp_find_logdev (pdev, tmp >> 16, tmp & 0xffff, 0);
+	acard->synth = isapnp_find_dev(card, tmp >> 16, tmp & 0xffff, NULL);
 	if (!acard->synth) {
 		snd_printk (LOGNAME "isapnp failed for Synth device\n");
 		return -ENODEV;
@@ -149,86 +145,77 @@ snd_wavefront_isapnp (int dev, snd_wavefront_card_t *acard, int pnp_id)
 
 	if (snd_cs4232_mpu_port[dev] != SND_AUTO_PORT) {
 		tmp = snd_wavefront_pnpids[2];
-		acard->mpu = isapnp_find_logdev (pdev, tmp >> 16, 
-						 tmp & 0xffff, 0);
+		acard->mpu = isapnp_find_dev(card, tmp >> 16, 
+					     tmp & 0xffff, NULL);
 		if (!acard->mpu) {
 			snd_printk ("isapnp failed for CS4232 MPU-401\n");
 			return -ENODEV;
 		}
 	}
 
-	if (isapnp_cfg_begin (acard->wss->dev->csn, acard->wss->number)<0)
-		return -EAGAIN;
-
 	/* PCM/FM initialization */
 
-	if (isapnp_config_init (&cfg, acard->wss)) {
-		isapnp_cfg_end();
+	pdev = acard->wss;
+	if (pdev->prepare(pdev)<0)
 		return -EAGAIN;
-	}
 
 	if (snd_cs4232_pcm_port[dev] != SND_AUTO_PORT)
-		cfg.port[0] = snd_cs4232_pcm_port[dev];
+		isapnp_resource_change(&pdev->resource[0], snd_cs4232_pcm_port[dev], 4);
 	if (snd_fm_port[dev] != SND_AUTO_PORT)
-		cfg.port[1] = snd_fm_port[dev];
+		isapnp_resource_change(&pdev->resource[1], snd_fm_port[dev], 4);
 	if (snd_wss_port[dev] != SND_AUTO_PORT)
-		cfg.port[2] = snd_wss_port[dev];
+		isapnp_resource_change(&pdev->resource[2], snd_wss_port[dev], 4);
 	if (snd_ics2115_port[dev] != SND_AUTO_PORT)
-		cfg.port[3] = snd_ics2115_port[dev];
+		isapnp_resource_change(&pdev->resource[3], snd_ics2115_port[dev], 8);
 
 	if (snd_dma1[dev] != SND_AUTO_DMA)
-		cfg.dma[0] = snd_dma1[dev];
+		isapnp_resource_change(&pdev->dma_resource[0], snd_dma1[dev], 1);
 	if (snd_dma2[dev] != SND_AUTO_DMA)
-		cfg.dma[1] = snd_dma2[dev] < 0 ? 254 : snd_dma2[dev];
+		isapnp_resource_change(&pdev->dma_resource[1], snd_dma2[dev], 1);
 
 	if (snd_cs4232_pcm_irq[dev] != SND_AUTO_IRQ)
-		cfg.irq[0] = snd_cs4232_pcm_irq[dev];
+		isapnp_resource_change(&pdev->irq_resource[0], snd_cs4232_pcm_irq[dev], 1);
 
-	if (isapnp_configure (&cfg)<0) {
+	if (pdev->activate(pdev)<0) {
 		snd_printk (LOGNAME "isapnp configuration failed\n");
-		isapnp_cfg_end();
 		return -EBUSY;
 	}
 
+	snd_cs4232_pcm_port[dev] = pdev->resource[0].start;
+	snd_fm_port[dev] = pdev->resource[1].start;
+	snd_wss_port[dev] = pdev->resource[2].start;
+	snd_ics2115_port[dev] = pdev->resource[4].start;
+
+	snd_dma1[dev] = pdev->dma_resource[0].start;
+	snd_dma2[dev] = pdev->dma_resource[1].start == 4 ? -1 : pdev->dma_resource[1].start;
+	snd_cs4232_pcm_irq[dev] = pdev->irq_resource[0].start;
+
 	snd_printk ("isapnp CS4232: port=0x%x, fm port=0x%x, wss port=0x%x\n",
-		    cfg.port[0], cfg.port[1], cfg.port[2]);
+		    snd_cs4232_pcm_port[dev], snd_fm_port[dev], snd_wss_port[dev]);
 	snd_printk ("isapnp CS4232: dma1=%i, dma2=%i, irq=%i\n",
-		    cfg.dma[0], cfg.dma[1], cfg.irq[0]);
-
-	snd_cs4232_pcm_port[dev] = cfg.port[0];
-	snd_fm_port[dev] = cfg.port[1];
-	snd_wss_port[dev] = cfg.port[2];
-	snd_ics2115_port[dev] = cfg.port[4];
-
-	snd_dma1[dev] = cfg.dma[0];
-	snd_dma2[dev] = cfg.dma[1] == 254 ? -1 : cfg.dma[1];
-	snd_cs4232_pcm_irq[dev] = cfg.irq[0];
+		    snd_dma1[dev], snd_dma2[dev], snd_cs4232_pcm_irq[dev]);
 
 	/* CTRL initialization */
 
 	if (snd_control_port[dev] != SND_AUTO_PORT) {
-		if (isapnp_config_init (&cfg, acard->ctrl)<0) {
-			isapnp_cfg_end();
+
+		pdev = acard->ctrl;
+
+		if (pdev->prepare(pdev)<0) {
+			acard->wss->deactivate(acard->wss);
 			return -EAGAIN;
 		}
 		
 		if (snd_control_port[dev] != SND_AUTO_PORT)
-			cfg.port[0] = snd_control_port[dev];
+			isapnp_resource_change(&pdev->resource[0], snd_control_port[dev], 8);
 		
-		cfg.port_disable[0] = snd_cs4232_pcm_port[dev];
-		cfg.port_disable_size[0] = 4;
-		cfg.port_disable[1] = snd_fm_port[dev];
-		cfg.port_disable_size[1] = 4;
-		cfg.port_disable[2] = snd_wss_port[dev];
-		cfg.port_disable_size[2] = 16;
-		
-		if (isapnp_configure(&cfg)<0) {
+		if (pdev->activate(pdev)<0) {
 			snd_printk(LOGNAME "isapnp configuration failed (step 2)\n");
-			isapnp_cfg_end();
+			acard->wss->deactivate(acard->wss);
 			return -EBUSY;
 		}
 		
-		snd_control_port[dev] = cfg.port[0];
+		snd_control_port[dev] = pdev->resource[0].start;
 		snd_printk ("isapnp CTRL: control port=0x%x\n", 
 			    snd_control_port[dev]);
 	}
@@ -237,35 +224,29 @@ snd_wavefront_isapnp (int dev, snd_wavefront_card_t *acard, int pnp_id)
 
 	if (snd_ics2115_port[dev] != SND_AUTO_PORT) {
 
-		if (isapnp_config_init (&cfg, acard->synth) <0) {
-			isapnp_cfg_end();
+		pdev = acard->synth;
+
+		if (pdev->prepare(pdev)<0) {
+			acard->wss->deactivate(acard->wss);
+			if (acard->ctrl)
+				acard->ctrl->deactivate(acard->ctrl);
 			return -EAGAIN;
 		}
 		
-		cfg.port[0] = snd_ics2115_port[dev];
-		if (snd_ics2115_irq[dev]  != SND_AUTO_IRQ) 
-			cfg.irq[0] = snd_ics2115_irq[dev];	
-		
-		cfg.port_disable[0] = snd_cs4232_pcm_port[dev];
-		cfg.port_disable_size[0] = 4;
-		cfg.port_disable[1] = snd_fm_port[dev];
-		cfg.port_disable_size[1] = 4;
-		cfg.port_disable[2] = snd_wss_port[dev];
-		cfg.port_disable_size[2] = 16;
+		isapnp_resource_change(&pdev->resource[0], snd_ics2115_port[dev], 8);
+		if (snd_ics2115_irq[dev] != SND_AUTO_IRQ) 
+			isapnp_resource_change(&pdev->irq_resource[0], snd_ics2115_irq[dev], 1);
 
-		if (snd_control_port[dev] != SND_AUTO_PORT) {
-			cfg.port_disable[3] = snd_control_port[dev];
-			cfg.port_disable_size[3] = 8;
-		}
-		
-		if (isapnp_configure (&cfg)<0) {
+		if (pdev->activate(pdev)<0) {
 			snd_printk(LOGNAME "isapnp synth configuration failed\n");
-			isapnp_cfg_end();
+			acard->wss->deactivate(acard->wss);
+			if (acard->ctrl)
+				acard->ctrl->deactivate(acard->ctrl);
 			return -EBUSY;
 		}
 		
-		snd_ics2115_port[dev] = cfg.port[0];
-		snd_ics2115_irq[dev] = cfg.irq[0];
+		snd_ics2115_port[dev] = pdev->resource[0].start;
+		snd_ics2115_irq[dev] = pdev->irq_resource[0].start;
 
 		snd_printk (LOGNAME "isapnp SYNTH: port=0x%x, irq=%i\n", 
 			    snd_ics2115_port[dev], 
@@ -275,47 +256,29 @@ snd_wavefront_isapnp (int dev, snd_wavefront_card_t *acard, int pnp_id)
 	/* MPU initialization */
 
 	if (snd_cs4232_mpu_port[dev] != SND_AUTO_PORT) {
-		int i;
 
-		if (isapnp_config_init(&cfg, acard->mpu)<0) {
-			isapnp_cfg_end();
+		pdev = acard->mpu;
+
+		if (pdev->prepare(pdev)<0) {
+			acard->wss->deactivate(acard->wss);
+			if (acard->ctrl)
+				acard->ctrl->deactivate(acard->ctrl);
+			if (acard->synth)
+				acard->synth->deactivate(acard->synth);
 			return -EAGAIN;
 		}
 
 		if (snd_cs4232_mpu_port[dev] != SND_AUTO_PORT)
-			cfg.port[0] = snd_cs4232_mpu_port[dev];
+			isapnp_resource_change(&pdev->resource[0], snd_cs4232_mpu_port[dev], 2);
 		if (snd_cs4232_mpu_irq[dev] != SND_AUTO_IRQ)
-			cfg.irq[0] = snd_cs4232_mpu_irq[dev];
+			isapnp_resource_change(&pdev->resource[0], snd_cs4232_mpu_irq[dev], 1);
 
-		cfg.port_disable[0] = snd_cs4232_pcm_port[dev];
-		cfg.port_disable_size[0] = 4;
-		cfg.port_disable[1] = snd_fm_port[dev];
-		cfg.port_disable_size[1] = 4;
-		cfg.port_disable[2] = snd_wss_port[dev];
-		cfg.port_disable_size[2] = 16;
-
-		i = 3;
-
-		if (snd_control_port[dev] != SND_AUTO_PORT) {
-			cfg.port_disable[i] = snd_control_port[dev];
-			cfg.port_disable_size[i] = 8;
-			i++;
-		}
-
-		cfg.irq_disable[0] = snd_cs4232_pcm_irq[dev];
-
-		if (snd_ics2115_port[dev] != SND_AUTO_PORT) {
-			cfg.port_disable[i] = snd_ics2115_port[dev];
-			cfg.port_disable_size[i] = 16;
-			cfg.irq_disable[1] = snd_ics2115_irq[dev];
-		}
-
-		if (isapnp_configure (&cfg)<0) {
+		if (pdev->activate(pdev)<0) {
 			snd_printk(LOGNAME "isapnp CS4232 MPU configuration failed\n");
 			snd_cs4232_mpu_port[dev] = SND_AUTO_PORT;
 		} else {
-			snd_cs4232_mpu_port[dev] = cfg.port[0];
-			snd_cs4232_mpu_irq[dev] = cfg.irq[0];
+			snd_cs4232_mpu_port[dev] = pdev->resource[0].start;
+			snd_cs4232_mpu_irq[dev] = pdev->irq_resource[0].start;
 		}
 
 		snd_printk (LOGNAME "isapnp MPU: port=0x%x, irq=%i\n", 
@@ -323,39 +286,20 @@ snd_wavefront_isapnp (int dev, snd_wavefront_card_t *acard, int pnp_id)
 			    snd_cs4232_mpu_irq[dev]);
 	}
 
-	isapnp_activate(acard->wss->number);
-
-	if (snd_control_port[dev] != SND_AUTO_PORT) {
-		isapnp_activate(acard->ctrl->number);
-	}
-
-	if (snd_control_port[dev] != SND_AUTO_PORT) {
-		isapnp_activate(acard->synth->number);
-	}
-
-	if (snd_cs4232_mpu_port[dev] != SND_AUTO_PORT) {
-		isapnp_activate(acard->mpu->number);
-	}
-
-	isapnp_cfg_end();
 	return 0;
 }
 
 static void 
 snd_wavefront_deactivate (snd_wavefront_card_t *acard)
 {
-	if (!acard->wss)
-		return;
-	if (isapnp_cfg_begin(acard->wss->dev->csn, acard->wss->number)<0)
-		return;
-	isapnp_deactivate(acard->wss->number);
+	if (acard->wss)
+		acard->wss->deactivate(acard->wss);
 	if (acard->ctrl)
-		isapnp_deactivate(acard->ctrl->number);
+		acard->ctrl->deactivate(acard->ctrl);
 	if (acard->mpu)
-		isapnp_deactivate(acard->mpu->number);
+		acard->mpu->deactivate(acard->mpu);
 	if (acard->synth)
-		isapnp_deactivate(acard->synth->number);
-	isapnp_cfg_end();
+		acard->synth->deactivate(acard->synth);
 }
 
 #endif CONFIG_ISAPNP
