@@ -33,10 +33,8 @@
 #define SND_RAWMIDI_MODE_STREAM	0x00000000	/* stream mode */
 #define SND_RAWMIDI_MODE_SEQ	0x00000001	/* sequencer mode */
 
-#define SND_RAWMIDI_FLG_SLEEP	0x00000001	/* process is sleeping */
-#define SND_RAWMIDI_FLG_FLUSH	0x00000002	/* flush in progress */
-#define SND_RAWMIDI_FLG_TRIGGER	0x00000004	/* trigger in progress */
-#define SND_RAWMIDI_FLG_TIMER	0x00000008	/* polling timer armed */
+#define SND_RAWMIDI_FLG_TRIGGER	0x00000001	/* trigger in progress */
+#define SND_RAWMIDI_FLG_TIMER	0x00000002	/* polling timer armed */
 #define SND_RAWMIDI_FLG_OSS	0x80000000	/* OSS compatible mode */
 
 #define SND_RAWMIDI_LFLG_OUTPUT	0x00000001	/* open for output */
@@ -120,13 +118,11 @@ struct snd_stru_rawmidi {
 	void *private_data;
 	void (*private_free) (void *private_data);
 
-	snd_spin_define(input);
-	snd_spin_define(input_sleep);
-	snd_spin_define(output);
-	snd_spin_define(output_sleep);
-	snd_sleep_define(input);
-	snd_sleep_define(output);
-	snd_mutex_define(open);
+	spinlock_t input_lock;
+	spinlock_t output_lock;
+	wait_queue_head_t input_sleep;
+	wait_queue_head_t output_sleep;
+	struct semaphore open_mutex;
 
 	snd_info_entry_t *dev;
 	snd_info_entry_t *proc_entry;
@@ -139,19 +135,23 @@ extern int snd_rawmidi_free(snd_rawmidi_t * rmidi);
 extern int __snd_rawmidi_register(snd_rawmidi_t * rmidi, int rawmidi_device);
 extern int __snd_rawmidi_unregister(snd_rawmidi_t * rmidi);
 #ifdef CONFIG_SND_SEQUENCER
-#include "seq_midi.h"
+#include "seq_device.h"
 static inline int snd_rawmidi_register(snd_rawmidi_t * rmidi, int rawmidi_device)
 {
 	int err;
 
 	if ((err = __snd_rawmidi_register(rmidi, rawmidi_device))<0)
 		return err;
-	snd_seq_midisynth_register_port(rmidi->card, rmidi->device, NULL);
+	snd_seq_device_register(rmidi->card, rmidi->device, NULL,
+				SND_SEQ_DEV_MIDISYNTH, NULL, 0, NULL);
 	return err;
 }
 static inline int snd_rawmidi_unregister(snd_rawmidi_t * rmidi)
 {
-	snd_seq_midisynth_unregister_port(rmidi->card, rmidi->device);
+	snd_seq_device_t *dev;
+	dev = snd_seq_device_find(rmidi->card, rmidi->device, SND_SEQ_DEV_MIDISYNTH);
+	if (dev)
+		snd_seq_device_unregister(dev);
 	return __snd_rawmidi_unregister(rmidi);
 }
 #else
