@@ -20,15 +20,16 @@
  */
 
 #define SND_MAIN_OBJECT_FILE
-#include "driver.h"
-#include "minors.h"
-#include "info.h"
-#include "sndpersist.h"
+#include "../include/driver.h"
+#include "../include/minors.h"
+#include "../include/info.h"
+#include "../include/control.h"
+#include "../include/sndpersist.h"
 #ifdef CONFIG_KMOD
 #include <linux/kmod.h>
 #endif
-#ifdef CONFIG_KERNELD
-#include <linux/kerneld.h>
+#ifdef CONFIG_DEVFS_FS
+#include <linux/devfs_fs_kernel.h>
 #endif
 
 #if 0
@@ -43,24 +44,16 @@
 #endif
 
 int snd_major = CONFIG_SND_MAJOR;
-#if !defined(LINUX_2_2) && defined(CONFIG_SND_OSSEMUL)
-int snd_oss_major = 14;		/* not configurable item at this time */
-#endif
 int snd_cards_limit = SND_CARDS, snd_ecards_limit;
 int snd_device_mode = S_IFCHR | S_IRUGO | S_IWUGO;
 int snd_device_gid = 0;
 int snd_device_uid = 0;
-#ifdef MODULE_PARM		/* hey - we have new 2.1.18+ kernel... */
 MODULE_AUTHOR("Jaroslav Kysela <perex@suse.cz>");
 MODULE_DESCRIPTION("Advanced Linux Sound Architecture driver for soundcards.");
 MODULE_SUPPORTED_DEVICE("sound");
 MODULE_PARM(snd_major, "i");
 MODULE_PARM_DESC(snd_major, "Major # for sound driver. (116 by default)");
 MODULE_PARM(snd_cards_limit, "1-" __MODULE_STRING(SND_CARDS) "i");
-#if !defined(LINUX_2_2) && defined(CONFIG_SND_OSSEMUL)
-MODULE_PARM_DESC(snd_major, "Major # for sound driver. (14 by default)");
-MODULE_PARM(snd_cards_limit, "1-" __MODULE_STRING(SND_CARDS) "i");
-#endif
 MODULE_PARM_DESC(snd_cards_limit, "Count of soundcards installed in the system.");
 MODULE_PARM(snd_device_mode, "i");
 MODULE_PARM_DESC(snd_device_mode, "Device file permission mask for sound dynamic device filesystem.");
@@ -68,14 +61,17 @@ MODULE_PARM(snd_device_gid, "i");
 MODULE_PARM_DESC(snd_device_gid, "Device file GID for sound dynamic device filesystem.");
 MODULE_PARM(snd_device_uid, "i");
 MODULE_PARM_DESC(snd_device_uid, "Device file UID for sound dynamic device filesystem.");
-#endif
 
 #define SND_OS_MINORS		256
 
 static int snd_minors_device[SND_OS_MINORS] = {[0 ... (32 - 1)] = -1};
 static snd_minor_t *snd_minors[SND_OS_MINORS] = {[0 ... (SND_OS_MINORS - 1)] = NULL};
 
-snd_mutex_define_static(sound);
+static DECLARE_MUTEX(sound_mutex);
+
+#ifdef CONFIG_DEVFS_FS
+static devfs_handle_t devfs_handle = NULL;
+#endif
 
 static inline snd_minor_t *snd_verify_card(unsigned short minor)
 {
@@ -89,18 +85,25 @@ static inline snd_minor_t *snd_verify_card(unsigned short minor)
 	return snd_minors[minor];
 }
 
-#if defined(CONFIG_KERNELD) || defined(CONFIG_KMOD)
+#ifdef CONFIG_KMOD
 
-static void snd_request_card(int card)
+void snd_request_card(int card)
 {
 	char str[32];
 
+	if (snd_cards_bitmap & (1 << card))
+		return;
 	if (card < 0 || card >= snd_ecards_limit)
 		return;
 	sprintf(str, "snd-card-%i", card);
 	request_module(str);
 }
 
+static void snd_request_sequencer(void)
+{
+	request_module("snd-seq");
+}
+
 #endif				/* request_module support */
 
 static inline snd_minor_t *snd_verify_card_open(unsigned short minor)
@@ -110,28 +113,23 @@ static inline snd_minor_t *snd_verify_card_open(unsigned short minor)
 	dev = minor & SND_MINOR_MASK;
 	if (dev != SND_MINOR_SEQUENCER) {
 		if (!(snd_cards_bitmap & (1 << (minor >> 5)))) {
-#if defined(CONFIG_KERNELD) || defined(CONFIG_KMOD)
+#ifdef CONFIG_KMOD
 			snd_request_card(minor >> 5);
 			if (!(snd_cards_bitmap & (1 << (minor >> 5))))
 #endif
 				return NULL;
 		}
+	} else {
+#ifdef CONFIG_KMOD
+		snd_request_sequencer();
+#endif
 	}
 	return snd_minors[minor];
 }
 
-#ifdef LINUX_2_1
 static loff_t snd_lseek(struct file *file, loff_t offset, int orig)
-#else
-static int snd_lseek(struct inode *inode, struct file *file,
-		     off_t offset, int orig)
-#endif				/* LINUX_2_1 */
 {
-#ifdef LINUX_2_1
 	unsigned short minor = MINOR(file->f_dentry->d_inode->i_rdev);
-#else
-	unsigned short minor = MINOR(inode->i_rdev);
-#endif
 	snd_minor_t *mptr;
 	snd_lseek_t *ptr;
 
@@ -143,20 +141,11 @@ static int snd_lseek(struct inode *inode, struct file *file,
 	DEBUG_RETURN("lseek", minor, ptr(file, offset, orig));
 }
 
-#ifdef LINUX_2_1
 static ssize_t snd_read(struct file *file,
                         char *buf, size_t count,
                         loff_t * offset)
-#else
-static int snd_read(struct inode *inode, struct file *file,
-                    char *buf, int count)
-#endif
 {
-#ifdef LINUX_2_1
 	unsigned short minor = MINOR(file->f_dentry->d_inode->i_rdev);
-#else
-	unsigned short minor = MINOR(inode->i_rdev);
-#endif
 	snd_minor_t *mptr;
 	snd_read_t *ptr;
 
@@ -168,20 +157,11 @@ static int snd_read(struct inode *inode, struct file *file,
 	DEBUG_RETURN("read", minor, ptr(file, buf, count));
 }
 
-#ifdef LINUX_2_1
 static ssize_t snd_write(struct file *file,
 			 const char *buf, size_t count,
 			 loff_t * offset)
-#else
-static int snd_write(struct inode *inode, struct file *file,
-		     const char *buf, int count)
-#endif
 {
-#ifdef LINUX_2_1
 	unsigned short minor = MINOR(file->f_dentry->d_inode->i_rdev);
-#else
-	unsigned short minor = MINOR(inode->i_rdev);
-#endif
 	snd_minor_t *mptr;
 	snd_write_t *ptr;
 
@@ -221,14 +201,9 @@ static int snd_release(struct inode *inode, struct file *file)
 	DEBUG_RETURN("release", minor, ptr(minor + 256, minor >> 5, snd_minors_device[minor], file));
 }
 
-#ifdef SND_POLL
 static unsigned int snd_poll(struct file *file, poll_table * wait)
 {
-#ifdef LINUX_2_1
 	unsigned short minor = MINOR(file->f_dentry->d_inode->i_rdev);
-#else
-	unsigned short minor = MINOR(file->f_inode->i_rdev);
-#endif
 	snd_minor_t *mptr;
 	snd_poll_t *ptr;
 
@@ -239,22 +214,6 @@ static unsigned int snd_poll(struct file *file, poll_table * wait)
 		return -ENODEV;
 	DEBUG_RETURN("poll", minor, ptr(file, wait));
 }
-#else
-static int snd_select(struct inode *inode, struct file *file,
-		      int sel_type, select_table * wait)
-{
-	unsigned short minor = MINOR(inode->i_rdev);
-	snd_minor_t *mptr;
-	snd_select_t *ptr;
-
-	DEBUG_ACCESS("select", minor);
-	if ((mptr = snd_verify_card(minor)) == NULL)
-		return -ENODEV;
-	if ((ptr = mptr->select) == NULL)
-		return -ENXIO;
-	DEBUG_RETURN("select", minor, ptr(file, sel_type, wait));
-}
-#endif
 
 static int snd_ioctl(struct inode *inode, struct file *file,
 		     unsigned int cmd, unsigned long arg)
@@ -271,19 +230,10 @@ static int snd_ioctl(struct inode *inode, struct file *file,
 	DEBUG_RETURN("ioctl", minor, ptr(file, cmd, arg));
 }
 
-#ifdef LINUX_2_1
 static int snd_mmap(struct file *file, struct vm_area_struct *vma)
-#else
-static int snd_mmap(struct inode *inode, struct file *file,
-                    struct vm_area_struct *vma)
-#endif
 {
-#ifdef LINUX_2_1
 	struct inode *inode = file->f_dentry->d_inode;
 	unsigned short minor = MINOR(inode->i_rdev);
-#else
-	unsigned short minor = MINOR(file->f_inode->i_rdev);
-#endif
 	snd_minor_t *mptr;
 	snd_mmap_t *ptr;
 
@@ -301,84 +251,45 @@ struct file_operations snd_fops =
 	snd_read,		/* read */
 	snd_write,		/* write */
 	NULL,			/* readdir */
-#ifdef SND_POLL
 	snd_poll,		/* poll */
-#else
-	snd_select,		/* select */
-#endif
 	snd_ioctl,		/* ioctl */
 	snd_mmap,		/* mmap */
 	snd_open,		/* open */
-#ifdef SND_FOPS_FLUSH
 	NULL,			/* flush */
-#endif
-#ifdef LINUX_2_1
 	snd_release,		/* release */
-#else
-	(void (*)(struct inode * inode, struct file * file)) snd_release,
-#endif
 	NULL,			/* fsync */
-#ifdef LINUX_2_1
 	NULL,			/* fasync */
 	NULL,			/* check_media_change */
 	NULL,			/* revalidate */
 	NULL,			/* lock */
-#endif
 };
 
-#ifndef LINUX_2_1
-
-int snd_ioctl_in(long *addr)
-{
-	if (verify_area(VERIFY_READ, addr, sizeof(int)) != 0) {
-		snd_printd("snd_ioctl_in: bad address 0x%lx\n", (long) addr);
-		return 0;
-	}
-	return get_fs_long(addr);	/* not really long - 32-bit int */
-}
-
-int snd_ioctl_out(long *addr, int value)
-{
-	int err;
-
-	if ((err = verify_area(VERIFY_WRITE, addr, sizeof(int))) != 0)
-		 return err;
-	put_fs_long(value, addr);	/* not really long - 32-bit int */
-	return 0;
-}
-
-#else
-
 int snd_ioctl_in(long *addr)
 {
 	int value;
 
-	if (verify_area(VERIFY_READ, addr, sizeof(int)) != 0) {
+	if (get_user(value, addr)) {
 		snd_printd("snd_ioctl_in: bad address 0x%lx\n", (long) addr);
 		return 0;
 	}
-	get_user(value, addr);
 	return value;
 }
 
 int snd_ioctl_out(long *addr, int value)
 {
-	int err;
-
-	if ((err = verify_area(VERIFY_WRITE, addr, sizeof(int))) != 0)
-		 return err;
-	put_user(value, addr);
+	if (put_user(value, addr))
+		return -EFAULT;
 	return 0;
 }
 
-#endif				/* !LINUX_2_1 */
-
 static void snd_init_minors(void)
 {
 	int idx;
 
 	for (idx = 0; idx <= SND_MINOR_MASK; idx++)
 		snd_minors_device[idx] = -1;
+	for (idx = 0; idx < SND_MINOR_HWDEPS; idx++)
+		snd_minors_device[idx + SND_MINOR_HWDEP] = idx;
 	for (idx = 0; idx < SND_MINOR_MIXERS; idx++)
 		snd_minors_device[idx + SND_MINOR_MIXER] = idx;
 	for (idx = 0; idx < SND_MINOR_RAWMIDIS; idx++)
@@ -401,6 +312,7 @@ static int snd_kernel_minor(int type, snd_card_t * card, int dev)
 			return -EINVAL;
 		minor = (card->number * (SND_MINOR_MASK + 1)) + type;
 		break;
+	case SND_DEVICE_TYPE_HWDEP:
 	case SND_DEVICE_TYPE_MIXER:
 	case SND_DEVICE_TYPE_RAWMIDI:
 	case SND_DEVICE_TYPE_PCM:
@@ -423,20 +335,21 @@ int snd_register_device(int type, snd_card_t * card, int dev, snd_minor_t * reg,
 
 	if (minor < 0)
 		return minor;
-	preg = (snd_minor_t *)snd_malloc(sizeof(snd_minor_t));
+	preg = (snd_minor_t *)snd_kmalloc(sizeof(snd_minor_t), GFP_KERNEL);
 	if (preg == NULL)
 		return -ENOMEM;
 	memcpy(preg, reg, sizeof(snd_minor_t));
 	preg->dev = NULL;
-	snd_mutex_down_static(sound);
+	down(&sound_mutex);
 	if (snd_minors[minor]) {
-		snd_mutex_up_static(sound);
-		snd_free(preg, sizeof(snd_minor_t));
+		up(&sound_mutex);
+		snd_kfree(preg);
 		return -EBUSY;
 	}
-	preg->dev = snd_info_create_device(name, minor, 0);
+	if (name)
+		preg->dev = snd_info_create_device(name, minor, 0);
 	snd_minors[minor] = preg;
-	snd_mutex_up_static(sound);
+	up(&sound_mutex);
 	return 0;
 }
 
@@ -447,16 +360,16 @@ int snd_unregister_device(int type, snd_card_t * card, int dev)
 
 	if (minor < 0)
 		return minor;
-	snd_mutex_down_static(sound);
+	down(&sound_mutex);
 	if ((mptr = snd_minors[minor]) == NULL) {
-		snd_mutex_up_static(sound);
+		up(&sound_mutex);
 		return -EINVAL;
 	}
 	if (mptr->dev)
 		snd_info_free_device(mptr->dev);
 	snd_minors[minor] = NULL;
-	snd_mutex_up_static(sound);
-	snd_free(mptr, sizeof(snd_minor_t));
+	up(&sound_mutex);
+	snd_kfree(mptr);
 	return 0;
 }
 
@@ -471,7 +384,7 @@ static void snd_minor_info_read(snd_info_buffer_t * buffer, void *private_data)
 	int idx, device;
 	snd_minor_t *mptr;
 
-	snd_mutex_down_static(sound);
+	down(&sound_mutex);
 	for (idx = 0; idx < SND_OS_MINORS; idx++) {
 		if ((mptr = snd_minors[idx]) == NULL)
 			continue;
@@ -484,7 +397,7 @@ static void snd_minor_info_read(snd_info_buffer_t * buffer, void *private_data)
 			snd_iprintf(buffer, "%3i:       : %s\n", idx, mptr->comment);
 		}
 	}
-	snd_mutex_up_static(sound);
+	up(&sound_mutex);
 }
 
 int snd_minor_info_init(void)
@@ -515,8 +428,6 @@ int snd_minor_info_done(void)
  *  persistant part
  */
 
-#if defined(LINUX_2_1) || LinuxVersionCode(2, 0, 34) <= LINUX_VERSION_CODE
-
 void (*snd_L_persist_lock) (int lock);
 int (*snd_L_persist_store) (char *key, const char *data, int data_len) = NULL;
 int (*snd_L_persist_restore) (char *key, char *data, int data_len) = NULL;
@@ -528,23 +439,21 @@ static void snd_persist_init(void)
 {
 	unsigned long flags;
 
-	snd_cli(&flags);
+	save_flags(flags);
+	cli();
 	snd_L_persist_lock = (void *) get_module_symbol(NULL, "persist_lock");
 	if (snd_L_persist_lock) {
 		snd_L_persist_lock(1);
 	} else {
-		snd_sti(&flags);
+		restore_flags(flags);
 		return;
 	}
-	snd_sti(&flags);
+	restore_flags(flags);
 	snd_L_persist_store = (void *) get_module_symbol(NULL, "persist_store");
 	snd_L_persist_restore = (void *) get_module_symbol(NULL, "persist_restore");
 	snd_L_persist_length = (void *) get_module_symbol(NULL, "persist_length");
 	snd_L_persist_present = (void *) get_module_symbol(NULL, "persist_present");
 	snd_L_persist_remove = (void *) get_module_symbol(NULL, "persist_remove");
-#if 0
-	printk("persist ok - remove = 0x%lx\n", (long) snd_L_persist_remove);
-#endif
 }
 
 static void snd_persist_done(void)
@@ -588,43 +497,42 @@ int snd_persist_remove(char *key)
 	return -ENXIO;
 }
 
-#endif
-
 /*
  *  INIT PART
  */
 
-#ifndef LINUX_2_1
-extern struct symbol_table snd_symbol_table_snd_export;
-#endif
-
 int init_module(void)
 {
+#ifdef CONFIG_DEVFS_FS
+	short controlnum;
+	char controlname[24];
+#endif
 #ifdef CONFIG_SND_OSSEMUL
 	int err;
 #endif
 
 	snd_ecards_limit = snd_cards_limit;
-#ifndef LINUX_2_1
-	if (register_symtab(&snd_symbol_table_snd_export) < 0)
-		return -ENOMEM;
-#endif
 	snd_init_minors();
 #ifdef CONFIG_SND_OSSEMUL
 	if ((err = snd_oss_init_module()) < 0)
 		return err;
 #endif
+#ifdef CONFIG_DEVFS_FS
+	devfs_handle = devfs_mk_dir("snd", 0);
+	if (devfs_register_chrdev(snd_major, "alsa", &snd_fops)) {
+#else
 	if (register_chrdev(snd_major, "alsa", &snd_fops)) {
+#endif
 		snd_printk("unable to register native major device number %d\n", snd_major);
 #ifdef CONFIG_SND_OSSEMUL
 		snd_oss_cleanup_module();
 #endif
 		return -EIO;
 	}
-	snd_malloc_init();
+	snd_memory_init();
 	snd_driver_init();
 	if (snd_info_init() < 0) {
-		snd_malloc_done();
+		snd_memory_done();
 #ifdef CONFIG_SND_OSSEMUL
 		snd_oss_cleanup_module();
 #endif
@@ -633,8 +541,14 @@ int init_module(void)
 #ifdef CONFIG_SND_OSSEMUL
 	snd_info_minor_register();
 #endif
-#if defined(LINUX_2_1) || LinuxVersionCode(2, 0, 34) <= LINUX_VERSION_CODE
 	snd_persist_init();
+#ifdef CONFIG_DEVFS_FS
+	for (controlnum = 0; controlnum < snd_cards_limit; controlnum++) {
+		sprintf(controlname, "snd/controlC%d", controlnum);
+		devfs_register(controlname , 0, DEVFS_FL_TOLERANT,
+				snd_major, controlnum<<5, snd_device_mode | S_IFCHR,
+				0, 0, &snd_fops, NULL);
+	}
 #endif
 	return 0;
 }
@@ -643,7 +557,18 @@ void cleanup_module(void)
 {
 	int idx;
 	snd_minor_t *minor;
+#ifdef CONFIG_DEVFS_FS
+	devfs_handle_t master;
+	char controlname[24];
+	short controlnum;
 
+	for (controlnum = 0; controlnum < snd_cards_limit; controlnum++) {
+		sprintf(controlname, "snd/controlC%d", controlnum);
+		master = devfs_find_handle(controlname, 0, 0, 0, DEVFS_SPECIAL_CHR, 0);
+		devfs_unregister(master);
+	}
+#endif
+	
 	for (idx = 0; idx < SND_OS_MINORS; idx++) {
 		minor = snd_minors[idx];
 		if (minor && minor->unregister)
@@ -653,7 +578,7 @@ void cleanup_module(void)
 			snd_minors[idx] = NULL;
 			if (minor->dev)
 				snd_info_free_device(minor->dev);
-			snd_free(minor, sizeof(snd_minor_t));
+			snd_kfree(minor);
 		}
 	}
 #ifdef CONFIG_SND_OSSEMUL
@@ -661,10 +586,110 @@ void cleanup_module(void)
 	snd_oss_cleanup_module();
 #endif
 	snd_info_done();
-	snd_malloc_done();
-#if defined(LINUX_2_1) || LinuxVersionCode(2, 0, 34) <= LINUX_VERSION_CODE
+	snd_memory_done();
 	snd_persist_done();
-#endif
+#ifdef CONFIG_DEVFS_FS
+	if (devfs_unregister_chrdev(snd_major, "alsa") != 0)
+#else
 	if (unregister_chrdev(snd_major, "alsa") != 0)
+#endif
 		snd_printk("unable to unregister major device number %d\n", snd_major);
+#ifdef CONFIG_DEVFS_FS
+	devfs_unregister(devfs_handle);
+#endif
 }
+
+  /* sound.c */
+EXPORT_SYMBOL(snd_ecards_limit);
+#if defined(CONFIG_KMOD)
+EXPORT_SYMBOL(snd_request_card);
+#endif
+EXPORT_SYMBOL(snd_register_device);
+EXPORT_SYMBOL(snd_unregister_device);
+#if defined(CONFIG_SND_OSSEMUL)
+EXPORT_SYMBOL(snd_register_oss_device);
+EXPORT_SYMBOL(snd_unregister_oss_device);
+#endif
+EXPORT_SYMBOL(snd_ioctl_in);
+EXPORT_SYMBOL(snd_ioctl_out);
+EXPORT_SYMBOL(snd_persist_length);
+EXPORT_SYMBOL(snd_persist_store);
+EXPORT_SYMBOL(snd_persist_restore);
+  /* memory.c */
+EXPORT_SYMBOL(snd_kmalloc);
+EXPORT_SYMBOL(snd_kcalloc);
+EXPORT_SYMBOL(snd_kfree);
+EXPORT_SYMBOL(snd_kmalloc_strdup);
+EXPORT_SYMBOL(snd_vmalloc);
+EXPORT_SYMBOL(snd_vfree);
+EXPORT_SYMBOL(snd_malloc_pages);
+EXPORT_SYMBOL(snd_free_pages);
+EXPORT_SYMBOL(snd_dma_malloc);
+EXPORT_SYMBOL(snd_dma_free);
+EXPORT_SYMBOL(snd_dma_notify_vma_close);
+  /* init.c */
+EXPORT_SYMBOL(snd_cards_count);
+EXPORT_SYMBOL(snd_cards_bitmap);
+EXPORT_SYMBOL(snd_cards);
+EXPORT_SYMBOL(snd_card_new);
+EXPORT_SYMBOL(snd_card_free);
+EXPORT_SYMBOL(snd_card_register);
+EXPORT_SYMBOL(snd_card_unregister);
+EXPORT_SYMBOL(snd_delay);
+EXPORT_SYMBOL(snd_check_ioport);
+EXPORT_SYMBOL(snd_register_ioport);
+EXPORT_SYMBOL(snd_unregister_ioport);
+EXPORT_SYMBOL(snd_unregister_ioports);
+EXPORT_SYMBOL(snd_register_dma_channel);
+EXPORT_SYMBOL(snd_unregister_dma_channels);
+EXPORT_SYMBOL(snd_register_interrupt);
+EXPORT_SYMBOL(snd_unregister_interrupts);
+  /* misc.c */
+EXPORT_SYMBOL(snd_dma_program);
+EXPORT_SYMBOL(snd_dma_disable);
+EXPORT_SYMBOL(snd_dma_residue);
+  /* info.c */
+EXPORT_SYMBOL(snd_iprintf);
+EXPORT_SYMBOL(snd_info_get_line);
+EXPORT_SYMBOL(snd_info_get_str);
+EXPORT_SYMBOL(snd_info_create_entry);
+EXPORT_SYMBOL(snd_info_free_entry);
+EXPORT_SYMBOL(snd_info_create_device);
+EXPORT_SYMBOL(snd_info_free_device);
+EXPORT_SYMBOL(snd_info_register);
+EXPORT_SYMBOL(snd_info_unregister);
+EXPORT_SYMBOL(snd_info_store_text);
+EXPORT_SYMBOL(snd_info_restore_text);
+  /* info_oss.c */
+#if defined(CONFIG_SND_OSSEMUL)
+EXPORT_SYMBOL(snd_oss_info_register);
+#endif
+  /* switch.c */
+EXPORT_SYMBOL(snd_switch_prepare);
+EXPORT_SYMBOL(snd_switch_new);
+EXPORT_SYMBOL(snd_switch_free_one);
+EXPORT_SYMBOL(snd_switch_add);
+EXPORT_SYMBOL(snd_switch_remove);
+EXPORT_SYMBOL(snd_switch_free);
+EXPORT_SYMBOL(snd_switch_lock);
+EXPORT_SYMBOL(snd_switch_list);
+EXPORT_SYMBOL(snd_switch_read);
+EXPORT_SYMBOL(snd_switch_write);
+EXPORT_SYMBOL(snd_switch_count);
+EXPORT_SYMBOL(snd_switch_size);
+EXPORT_SYMBOL(snd_switch_store);
+EXPORT_SYMBOL(snd_switch_restore);
+  /* control.c */
+EXPORT_SYMBOL(snd_control_busy);
+EXPORT_SYMBOL(snd_control_notify_structure_change);
+EXPORT_SYMBOL(snd_control_notify_value_change);
+EXPORT_SYMBOL(snd_control_notify_switch_change);
+EXPORT_SYMBOL(snd_control_notify_switch_value_change);
+EXPORT_SYMBOL(snd_control_register_ioctl);
+EXPORT_SYMBOL(snd_control_unregister_ioctl);
+EXPORT_SYMBOL(snd_control_switch_add);
+EXPORT_SYMBOL(snd_control_switch_remove);
+EXPORT_SYMBOL(snd_control_switch_new);
+EXPORT_SYMBOL(snd_control_switch_change);
+  /* misc.c */
+EXPORT_SYMBOL(snd_task_name);
