@@ -20,11 +20,11 @@
  */
 
 #define SND_MAIN_OBJECT_FILE
-#include "driver.h"
-#include "timer.h"
-#include "control.h"
-#include "info.h"
-#include "minors.h"
+#include "../include/driver.h"
+#include "../include/timer.h"
+#include "../include/control.h"
+#include "../include/info.h"
+#include "../include/minors.h"
 #ifdef CONFIG_KMOD
 #include <linux/kmod.h>
 #endif
@@ -33,10 +33,8 @@
 #endif
 
 int snd_timer_limit = SND_TIMER_GLOBAL_SYSTEM + 1;
-#ifdef MODULE_PARM
 MODULE_PARM(snd_timer_limit, "i");
 MODULE_PARM_DESC(snd_timer_limit, "Maximum global timers in system. (1 by default)");
-#endif
 
 typedef struct {
 	snd_timer_instance_t *timeri;
@@ -47,15 +45,15 @@ typedef struct {
 	int qused;
 	int queue_size;
 	snd_timer_read_t *queue;
-	snd_spin_define(qlock);
-	snd_sleep_define(qchange);
+	spinlock_t qlock;
+	wait_queue_head_t qchange_sleep;
 } snd_timer_user_t;
 
 snd_timer_t *snd_timer_devices = NULL;
 snd_timer_instance_t *snd_timer_slave_devices = NULL;
 
-snd_spin_define_static(snd_timer_slave);
-snd_mutex_define_static(register);
+static spinlock_t snd_timer_slave = SPIN_LOCK_UNLOCKED;
+static DECLARE_MUTEX(register_mutex);
 
 static void snd_timer_reschedule(snd_timer_t * timer, unsigned long ticks_left);
 
@@ -72,17 +70,17 @@ static snd_timer_instance_t *__snd_timer_open(char *owner, snd_timer_t *timer,
 	
 	if (!timer)
 		return NULL;
-	timeri = snd_calloc(sizeof(*timeri));
+	timeri = snd_kcalloc(sizeof(*timeri), GFP_KERNEL);
 	if (!timeri)
 		return NULL;
 	MOD_INC_USE_COUNT;
 	if (timer->card)
 		timer->card->use_inc(timer->card);
 	timeri->timer = timer;
-	timeri->owner = snd_malloc_strdup(owner);
+	timeri->owner = snd_kmalloc_strdup(owner, GFP_KERNEL);
 	timeri->slave_type = slave_type;
 	timeri->slave_id = slave_id;
-	snd_spin_lock(timer, lock, &flags);
+	spin_lock_irqsave(&timer->lock, flags);
 	if (timer->first != NULL) {
 		timeri->next = timer->first;
 	} else {
@@ -90,7 +88,7 @@ static snd_timer_instance_t *__snd_timer_open(char *owner, snd_timer_t *timer,
 		timer->hw.open(timer);
 	}
 	timer->first = timeri;
-	snd_spin_unlock(timer, lock, &flags);
+	spin_unlock_irqrestore(&timer->lock, flags);
 	return timeri;
 }
 
@@ -145,19 +143,19 @@ snd_timer_instance_t *snd_timer_open(char *owner, int timer_no,
 	snd_timer_t *timer;
 	snd_timer_instance_t *timeri = NULL;
 	
-	snd_mutex_down_static(register);
+	down(&register_mutex);
 	timer = snd_timer_find(timer_no);
-#if defined(CONFIG_KERNELD) || defined(CONFIG_KMOD)
+#ifdef CONFIG_KMOD
 	if (timer == NULL) {
-		snd_mutex_up_static(register);
+		up(&register_mutex);
 		snd_timer_request(timer_no);
-		snd_mutex_down_static(register);
+		down(&register_mutex);
 		timer = snd_timer_find(timer_no);
 	}
 #endif
 	if (timer)
 		timeri = __snd_timer_open(owner, timer, slave_type, slave_id);
-	snd_mutex_up_static(register);
+	up(&register_mutex);
 	return timeri;
 }
 
@@ -167,9 +165,9 @@ snd_timer_instance_t *snd_timer_open1(char *owner, snd_timer_t *timer,
 {
 	snd_timer_instance_t *timeri;
 
-	snd_mutex_down_static(register);
+	down(&register_mutex);
 	timeri = __snd_timer_open(owner, timer, slave_type, slave_id);
-	snd_mutex_up_static(register);
+	up(&register_mutex);
 	return timeri;
 }
 
@@ -180,21 +178,21 @@ snd_timer_instance_t *snd_timer_open_slave(char *owner,
 	snd_timer_instance_t *timeri;
 	unsigned long flags;
 	
-	timeri = snd_calloc(sizeof(*timeri));
+	timeri = snd_kcalloc(sizeof(*timeri), GFP_KERNEL);
 	if (!timeri)
 		return NULL;
 	MOD_INC_USE_COUNT;
-	timeri->owner = snd_malloc_strdup(owner);
+	timeri->owner = snd_kmalloc_strdup(owner, GFP_KERNEL);
 	timeri->slave_type = slave_type;
 	timeri->slave_id = slave_id;
-	snd_spin_lock_static(snd_timer_slave, &flags);
+	spin_lock_irqsave(&snd_timer_slave, flags);
 	timeri->flags |= SND_TIMER_IFLG_SLAVE;
 	if (snd_timer_slave_devices != NULL)
 		timeri->next = snd_timer_slave_devices;
 	snd_timer_slave_devices = timeri;
 	timeri->slave_type = slave_type;
 	timeri->slave_id = slave_id;
-	snd_spin_unlock_static(snd_timer_slave, &flags);
+	spin_unlock_irqrestore(&snd_timer_slave, flags);
 	return timeri;
 }
 
@@ -210,14 +208,17 @@ int snd_timer_close(snd_timer_instance_t * timeri)
 	if (!(timeri->flags & SND_TIMER_IFLG_SLAVE)) {
 		if ((timer = timeri->timer) == NULL)
 			return -EINVAL;
-		snd_spin_lock(timer, lock, &flags);
+		spin_lock_irqsave(&timer->lock, flags);
 		if ((timeri1 = timer->first) == timeri) {
 			timer->first = timeri->next;
 		} else {
-			while (timeri1 && timeri1->next != timeri)
+			while (timeri1) {
+				if (timeri1->next == timeri)
+					break;
 				timeri1 = timeri1->next;
+			}
 			if (!timeri1) {
-				snd_spin_unlock(timer, lock, &flags);
+				spin_unlock_irqrestore(&timer->lock, flags);
 				return -EINVAL;
 			}
 			timeri1->next = timeri->next;
@@ -226,26 +227,29 @@ int snd_timer_close(snd_timer_instance_t * timeri)
 			if (timer->hw.close)
 				timer->hw.close(timer);
 		}
-		snd_spin_unlock(timer, lock, &flags);
+		spin_unlock_irqrestore(&timer->lock, flags);
 	} else {
-		snd_spin_lock_static(snd_timer_slave, &flags);
+		spin_lock_irqsave(&snd_timer_slave, flags);
 		if ((timeri1 = snd_timer_slave_devices) == timeri) {
 			snd_timer_slave_devices = timeri->next;
 		} else {
-			while (timeri1 && timeri1->next != timeri)
+			while (timeri1) {
+				if (timeri1->next == timeri)
+					break;
 				timeri1 = timeri1->next;
+			}
 			if (!timeri1) {
-				snd_spin_unlock_static(snd_timer_slave, &flags);
+				spin_unlock_irqrestore(&snd_timer_slave, flags);
 				return -EINVAL;
 			}
 			timeri1->next = timeri->next;
 		}
-		snd_spin_unlock_static(snd_timer_slave, &flags);
+		spin_unlock_irqrestore(&snd_timer_slave, flags);
 	}
 	if (timeri->private_free)
 		timeri->private_free(timeri->private_data);
-	snd_free_str(timeri->owner);
-	snd_free(timeri, sizeof(*timeri));
+	snd_kfree(timeri->owner);
+	snd_kfree(timeri);
 	MOD_DEC_USE_COUNT;
 	if (timer && timer->card)
 		timer->card->use_dec(timer->card);
@@ -276,7 +280,7 @@ int snd_timer_start(snd_timer_instance_t * timeri, unsigned int ticks)
 		return result;
 	if (!(timeri->flags & SND_TIMER_IFLG_SLAVE)) {
 		if ((timer = timeri->timer) != NULL) {
-			snd_spin_lock(timer, lock, &flags);
+			spin_lock_irqsave(&timer->lock, flags);
 			timeri->ticks = timeri->cticks = ticks;
 			if (timer->running) {
 				timer->flags |= SND_TIMER_FLG_RESCHED;
@@ -289,12 +293,12 @@ int snd_timer_start(snd_timer_instance_t * timeri, unsigned int ticks)
 				timeri->flags |= SND_TIMER_IFLG_RUNNING;
 				result = 0;
 			}
-			snd_spin_unlock(timer, lock, &flags);
+			spin_unlock_irqrestore(&timer->lock, flags);
 		}
 	} else {
-		snd_spin_lock_static(snd_timer_slave, &flags);
+		spin_lock_irqsave(&snd_timer_slave, flags);
 		timeri->flags |= SND_TIMER_IFLG_RUNNING;
-		snd_spin_unlock_static(snd_timer_slave, &flags);
+		spin_unlock_irqrestore(&snd_timer_slave, flags);
 		result = 0;
 	}
 	return result;
@@ -309,7 +313,7 @@ int snd_timer_stop(snd_timer_instance_t * timeri)
 		return -EINVAL;
 	if (!(timeri->flags & SND_TIMER_IFLG_SLAVE)) {
 		if ((timer = timeri->timer) != NULL) {
-			snd_spin_lock(timer, lock, &flags);
+			spin_lock_irqsave(&timer->lock, flags);
 			if (timeri->flags & SND_TIMER_IFLG_RUNNING) {
 				timeri->flags &= ~SND_TIMER_IFLG_RUNNING;
 				if (!(--timer->running)) {
@@ -324,13 +328,13 @@ int snd_timer_stop(snd_timer_instance_t * timeri)
 					}
 				}
 			}		
-			snd_spin_unlock(timer, lock, &flags);
+			spin_unlock_irqrestore(&timer->lock, flags);
 			return 0;
 		}
 	} else {
-		snd_spin_lock_static(snd_timer_slave, &flags);
+		spin_lock_irqsave(&snd_timer_slave, flags);
 		timeri->flags &= ~SND_TIMER_IFLG_RUNNING;
-		snd_spin_unlock_static(snd_timer_slave, &flags);
+		spin_unlock_irqrestore(&snd_timer_slave, flags);
 		return 0;
 	}
 	return -EINVAL;
@@ -347,7 +351,7 @@ int snd_timer_continue(snd_timer_instance_t * timeri)
 	if (timeri->flags & SND_TIMER_IFLG_SLAVE)
 		return snd_timer_start(timeri, 1);
 	if ((timer = timeri->timer) != NULL) {
-		snd_spin_lock(timer, lock, &flags);
+		spin_lock_irqsave(&timer->lock, flags);
 		if (!timer->running) {
 			if (!timeri->cticks)
 				timeri->cticks = 1;
@@ -362,7 +366,7 @@ int snd_timer_continue(snd_timer_instance_t * timeri)
 			timeri->flags |= SND_TIMER_IFLG_START;
 			result = 1;	/* delayed start */
 		}
-		snd_spin_unlock(timer, lock, &flags);
+		spin_unlock_irqrestore(&timer->lock, flags);
 	}
 	return result;
 }
@@ -424,7 +428,7 @@ void snd_timer_interrupt(snd_timer_t * timer, unsigned long ticks_left)
 		return;
 	ts.iprev = NULL;
 	ts.inext = NULL;
-	snd_spin_lock(timer, lock, &flags);
+	spin_lock_irqsave(&timer->lock, flags);
 	ti = timer->first;
 	while (ti) {
 		if (ti->flags & SND_TIMER_IFLG_RUNNING) {
@@ -485,7 +489,7 @@ void snd_timer_interrupt(snd_timer_t * timer, unsigned long ticks_left)
 		ti->iprev = ti->inext = NULL;
 		tx.inext = tx.iprev = NULL;
 		tp = NULL;
-		snd_spin_lock_static(snd_timer_slave, &sflags);
+		spin_lock_irqsave(&snd_timer_slave, sflags);
 		tc = snd_timer_slave_devices;
 		while (tc != NULL) {
 			if ((tc->flags & SND_TIMER_IFLG_RUNNING) &&
@@ -509,26 +513,26 @@ void snd_timer_interrupt(snd_timer_t * timer, unsigned long ticks_left)
 			}
 			tc = tc->next;
 		}
-		snd_spin_unlock_static(snd_timer_slave, &sflags);
+		spin_unlock_irqrestore(&snd_timer_slave, sflags);
 		ticks = ti->ticks;
 		resolution = snd_timer_resolution(ti);
-		snd_spin_unlock(timer, lock, &flags);
+		spin_unlock_irqrestore(&timer->lock, flags);
 		if (ti->callback)
 			ti->callback(ti, resolution, ticks, ti->callback_data);
 		while (tx.inext) {
-			snd_spin_lock_static(snd_timer_slave, &sflags);
+			spin_lock_irqsave(&snd_timer_slave, sflags);
 			tc = tx.inext;
 			tx.inext = tc->inext;
 			if (tx.inext)
 				tx.inext->iprev = &tx;
 			tc->iprev = tc->inext = NULL;
-			snd_spin_unlock_static(snd_timer_slave, &sflags);
+			spin_unlock_irqrestore(&snd_timer_slave, sflags);
 			if (tc->callback)
 				tc->callback(tc, resolution, ticks, tc->callback_data);
 		}
-		snd_spin_lock(timer, lock, &flags);
+		spin_lock_irqsave(&timer->lock, flags);
 	}
-	snd_spin_unlock(timer, lock, &flags);
+	spin_unlock_irqrestore(&timer->lock, flags);
 }
 
 /*
@@ -539,14 +543,14 @@ snd_timer_t *snd_timer_new_device(snd_card_t * card, char *id)
 {
 	snd_timer_t *timer;
 
-	timer = (snd_timer_t *) snd_calloc(sizeof(snd_timer_t));
+	timer = (snd_timer_t *) snd_kcalloc(sizeof(snd_timer_t), GFP_KERNEL);
 	if (!timer)
 		return NULL;
 	timer->card = card;
 	if (id) {
 		strncpy(timer->id, id, sizeof(timer->id) - 1);
 	}
-	snd_spin_prepare(timer, lock);
+	timer->lock = SPIN_LOCK_UNLOCKED;
 	return timer;
 }
 
@@ -556,7 +560,7 @@ int snd_timer_free(snd_timer_t * timer)
 		return -EINVAL;
 	if (timer->private_free)
 		timer->private_free(timer->private_data);
-	snd_free(timer, sizeof(*timer));
+	snd_kfree(timer);
 	return 0;
 }
 
@@ -585,18 +589,21 @@ int snd_timer_register(snd_timer_t * timer, int device)
 			device |= SND_TIMER_TYPE_GLOBAL;
 		}
 	}
+
 	timer->number = device;
-	snd_mutex_down_static(register);
+	down(&register_mutex);
 	if ((timer1 = snd_timer_devices) == NULL) {
 		snd_timer_devices = timer;
 	} else {
 		timer2 = NULL;
-		while (timer1 && timer1->number < device) {
+		while (timer1) {
+			if (timer1->number >= device)
+				break;
 			timer2 = timer1;
 			timer1 = timer1->next;
 		}
 		if (timer1 && timer1->number == device) {
-			snd_mutex_up_static(register);
+			up(&register_mutex);
 			return -EBUSY;
 		}
 		if (timer2) {
@@ -607,7 +614,7 @@ int snd_timer_register(snd_timer_t * timer, int device)
 			snd_timer_devices = timer;
 		}
 	}
-	snd_mutex_up_static(register);
+	up(&register_mutex);
 	return 0;
 }
 
@@ -621,21 +628,27 @@ int snd_timer_unregister(snd_timer_t * timer)
 		snd_printd("timer 0x%lx is busy?\n", (long)timer);
 		return -EBUSY;
 	}
-	snd_mutex_down_static(register);
+	
+	down(&register_mutex);
 	if ((timer1 = snd_timer_devices) == timer) {
 		snd_timer_devices = timer->next;
 	} else {
-		while (timer1 && timer1->next != timer)
+		while (timer1) {
+			if (timer1->next == timer)
+				break;
 			timer1 = timer1->next;
+		}
 		if (timer1 == NULL) {
-			snd_mutex_up_static(register);
+			up(&register_mutex);
 			return -EINVAL;
+		} else {
+			timer1->next = timer->next;			
 		}
 	}
-	snd_mutex_up_static(register);
+	up(&register_mutex);
 	if (timer->private_free)
 		timer->private_free(timer->private_data);
-	snd_free(timer, sizeof(*timer));
+	snd_kfree(timer);
 	return 0;
 }
 
@@ -687,7 +700,7 @@ static struct snd_stru_timer_hardware snd_timer_system =
 static void snd_timer_free_system(void *private_data)
 {
 	if (private_data)
-		snd_free(private_data, sizeof(struct timer_list));
+		snd_kfree(private_data);
 }
 
 static int snd_timer_register_system(void)
@@ -700,7 +713,7 @@ static int snd_timer_register_system(void)
 		return -ENOMEM;
 	strcpy(timer->name, "system timer");
 	memcpy(&timer->hw, &snd_timer_system, sizeof(snd_timer_system));
-	tlist = (struct timer_list *) snd_calloc(sizeof(struct timer_list));
+	tlist = (struct timer_list *) snd_kcalloc(sizeof(struct timer_list), GFP_KERNEL);
 	if (!tlist) {
 		snd_timer_free(timer);
 		return -ENOMEM;
@@ -722,7 +735,7 @@ static void snd_timer_proc_read(snd_info_buffer_t * buffer, void *private_data)
 	snd_timer_t *timer;
 	snd_timer_instance_t *ti;
 
-	snd_mutex_down_static(register);
+	down(&register_mutex);
 	timer = snd_timer_devices;
 	while (timer) {
 		switch (timer->number & SND_TIMER_TYPE_MAX) {
@@ -750,7 +763,7 @@ static void snd_timer_proc_read(snd_info_buffer_t * buffer, void *private_data)
 		if (timer->hw.flags & SND_TIMER_HW_SLAVE)
 			snd_iprintf(buffer, " SLAVE");
 		snd_iprintf(buffer, "\n");
-		snd_spin_lock(timer, lock, &flags);
+		spin_lock_irqsave(&timer->lock, flags);
 		if (timer->first) {
 			ti = timer->first;
 			while (ti) {
@@ -761,10 +774,10 @@ static void snd_timer_proc_read(snd_info_buffer_t * buffer, void *private_data)
 				ti = ti->next;
 			}
 		}
-		snd_spin_unlock(timer, lock, &flags);
+		spin_unlock_irqrestore(&timer->lock, flags);
 		timer = timer->next;
 	}
-	snd_mutex_up_static(register);
+	up(&register_mutex);
 }
 
 /*
@@ -782,7 +795,7 @@ static int snd_timer_control_ioctl(snd_card_t * card, snd_control_t * control,
 			struct snd_ctl_hw_info *ptr = (struct snd_ctl_hw_info *) arg;
 		
 			dev = (card->number * 32) + 256;
-			snd_mutex_down_static(register);
+			down(&register_mutex);
 			timer = snd_timer_devices;
 			ptr->timerdevs = 0;
 			while (timer) {
@@ -792,7 +805,7 @@ static int snd_timer_control_ioctl(snd_card_t * card, snd_control_t * control,
 					ptr->timerdevs++;
 				timer = timer->next;
 			}
-			snd_mutex_up_static(register);
+			up(&register_mutex);
 			return 0;
 		}
 	}
@@ -815,19 +828,14 @@ static void snd_timer_user_interrupt(snd_timer_instance_t *timeri,
 	if (tu->qused >= tu->queue_size) {
 		tu->overrun++;
 	} else {
-		snd_spin_lock(tu, qlock, &flags);
+		spin_lock_irqsave(&tu->qlock, flags);
 		r = &tu->queue[tu->qtail++];
 		tu->qtail %= tu->queue_size;
 		r->resolution = resolution;
 		r->ticks = ticks;
 		tu->qused++;
-		if (snd_getlock(tu, qchange) & SND_WK_SLEEP) {
-			snd_getlock(tu, qchange) &= ~SND_WK_SLEEP;
-			snd_spin_unlock(tu, qlock, &flags);
-			snd_wakeup(tu, qchange);
-		} else {
-			snd_spin_unlock(tu, qlock, &flags);
-		}
+		spin_unlock_irqrestore(&tu->qlock, flags);
+		wake_up(&tu->qchange_sleep);
 	}
 }
 
@@ -836,16 +844,16 @@ static int snd_timer_user_open(unsigned short minor, int cardnum, int device,
 {
 	snd_timer_user_t *tu;
 	
-	tu = (snd_timer_user_t *)snd_calloc(sizeof(*tu));
+	tu = (snd_timer_user_t *)snd_kcalloc(sizeof(*tu), GFP_KERNEL);
 	if (!tu)
 		return -ENOMEM;
-	snd_spin_prepare(tu, qlock);
-	snd_sleep_prepare(tu, qchange);
+	tu->qlock = SPIN_LOCK_UNLOCKED;
+	init_waitqueue_head(&tu->qchange_sleep);
 	tu->ticks = 1;
 	tu->queue_size = 128;
-	tu->queue = (snd_timer_read_t *)snd_malloc(tu->queue_size * sizeof(snd_timer_read_t));
+	tu->queue = (snd_timer_read_t *)snd_kmalloc(tu->queue_size * sizeof(snd_timer_read_t), GFP_KERNEL);
 	if (!tu->queue) {
-		snd_free(tu, sizeof(*tu));
+		snd_kfree(tu);
 		return -ENOMEM;
 	}
 	file->private_data = tu;
@@ -864,8 +872,8 @@ static int snd_timer_user_release(unsigned short minor, int cardnum, int device,
 		if (tu->timeri)
 			snd_timer_close(tu->timeri);
 		if (tu->queue)
-			snd_free(tu->queue, tu->queue_size * sizeof(snd_timer_read_t));
-		snd_free(tu, sizeof(*tu));
+			snd_kfree(tu->queue);
+		snd_kfree(tu);
 	}
 	MOD_DEC_USE_COUNT;
 	return 0;
@@ -876,11 +884,9 @@ static int snd_timer_user_general_info(snd_timer_general_info_t *_ginfo)
 	snd_timer_general_info_t ginfo;
 	snd_timer_t *timer;
 	
-	if (verify_area(VERIFY_WRITE, (void *) _ginfo, sizeof(ginfo)))
-		return -EFAULT;
 	memset(&ginfo, 0, sizeof(ginfo));
 	ginfo.count = snd_timer_limit;
-	snd_mutex_down_static(register);
+	down(&register_mutex);
 	timer = snd_timer_devices;
 	while (timer) {
 		if (timer->number >= 256)
@@ -889,8 +895,9 @@ static int snd_timer_user_general_info(snd_timer_general_info_t *_ginfo)
 			ginfo.count = timer->number + 1;
 		timer = timer->next;
 	}
-	snd_mutex_up_static(register);
-	copy_to_user(_ginfo, &ginfo, sizeof(*_ginfo));
+	up(&register_mutex);
+	if (copy_to_user(_ginfo, &ginfo, sizeof(*_ginfo)))
+		return -EFAULT;
 	return 0;
 } 
 
@@ -905,9 +912,8 @@ static int snd_timer_user_tselect(struct file *file, snd_timer_select_t *_tselec
 		return -EINVAL;
 	if (tu->timeri)
 		snd_timer_close(tu->timeri);
-	if (verify_area(VERIFY_READ, _tselect, sizeof(*_tselect)))
+	if (copy_from_user(&tselect, _tselect, sizeof(tselect)))
 		return -EFAULT;
-	copy_from_user(&tselect, _tselect, sizeof(tselect));
 	if (!tselect.slave && tselect.data.number < 0)
 		return -EINVAL;
 	if (tselect.slave && tselect.data.slave.type == SND_TIMER_STYPE_NONE)
@@ -940,8 +946,6 @@ static int snd_timer_user_info(struct file *file, snd_timer_info_t *_info)
 	t = tu->timeri->timer;
 	if (!t)
 		return -EINVAL;
-	if (verify_area(VERIFY_WRITE, _info, sizeof(*_info)))
-		return -EFAULT;
 	memset(&info, 0, sizeof(info));
 	if (t->hw.flags & SND_TIMER_HW_SLAVE)
 		info.flags |= SND_TIMER_FLG_SLAVE;
@@ -949,7 +953,8 @@ static int snd_timer_user_info(struct file *file, snd_timer_info_t *_info)
 	strncpy(info.name, t->name, sizeof(info.name)-1);
 	info.ticks = t->hw.ticks;
 	info.resolution = t->hw.resolution;
-	copy_to_user(_info, &info, sizeof(*_info));
+	if (copy_to_user(_info, &info, sizeof(*_info)))
+		return -EFAULT;
 	return 0;
 }
 
@@ -967,25 +972,24 @@ static int snd_timer_user_params(struct file *file, snd_timer_params_t *_params)
 	t = tu->timeri->timer;
 	if (!t)
 		return -EINVAL;
-	if (verify_area(VERIFY_READ, _params, sizeof(params)))
+	if (copy_from_user(&params, _params, sizeof(params)))
 		return -EFAULT;
-	copy_from_user(&params, _params, sizeof(params));
 	if (!(t->hw.flags & SND_TIMER_HW_SLAVE) && params.ticks < 1)
 		return -EINVAL;
 	if (params.queue_size > 0 && (params.queue_size < 32 || params.queue_size > 1024))
 		return -EINVAL;
 	snd_timer_stop(tu->timeri);
-	snd_spin_lock(t, lock, &flags);
+	spin_lock_irqsave(&t->lock, flags);
 	if (params.flags & SND_TIMER_PSFLG_AUTO) {
 		tu->timeri->flags |= SND_TIMER_IFLG_AUTO;
 	} else {
 		tu->timeri->flags &= ~SND_TIMER_IFLG_AUTO;
 	}
-	snd_spin_unlock(t, lock, &flags);
+	spin_unlock_irqrestore(&t->lock, flags);
 	if (params.queue_size > 0 && tu->queue_size != params.queue_size) {
-		tr = (snd_timer_read_t *)snd_malloc(params.queue_size * sizeof(snd_timer_read_t));
+		tr = (snd_timer_read_t *)snd_kmalloc(params.queue_size * sizeof(snd_timer_read_t), GFP_KERNEL);
 		if (tr) {
-			snd_free(tu->queue, tu->queue_size * sizeof(snd_timer_read_t));
+			snd_kfree(tu->queue);
 			tu->queue_size = params.queue_size;
 			tu->queue = tr;
 		}
@@ -1007,17 +1011,16 @@ static int snd_timer_user_status(struct file *file, snd_timer_status_t *_status)
 	tu = (snd_timer_user_t *)file->private_data;
 	if (!tu || !tu->timeri)
 		return -EINVAL;
-	if (verify_area(VERIFY_WRITE, _status, sizeof(*_status)))
-		return -EFAULT;
 	memset(&status, 0, sizeof(status));
 	status.resolution = snd_timer_resolution(tu->timeri);
 	status.lost = tu->timeri->lost;
 	status.overrun = tu->overrun;
-	snd_spin_lock(tu, qlock, &flags);
+	spin_lock_irqsave(&tu->qlock, flags);
 	status.queue_size = tu->queue_size;
 	status.queue = tu->qused;
-	snd_spin_unlock(tu, qlock, &flags);
-	copy_to_user(_status, &status, sizeof(status));
+	spin_unlock_irqrestore(&tu->qlock, flags);
+	if (copy_to_user(_status, &status, sizeof(status)))
+		return -EFAULT;
 	return 0;
 }
 
@@ -1099,25 +1102,26 @@ static long snd_timer_user_read(struct file *file, char *buffer, long count)
 	if (!tu)
 		return -EINVAL;
 	while (count - result >= sizeof(snd_timer_read_t)) {
-		snd_spin_lock(tu, qlock, &flags);
+		spin_lock_irqsave(&tu->qlock, flags);
 		if (!tu->qused) {
-			snd_spin_unlock(tu, qlock, &flags);
+			spin_unlock_irqrestore(&tu->qlock, flags);
 			break;
 		}
-		copy_to_user(buffer, &tu->queue[tu->qhead++], sizeof(snd_timer_read_t));
+		if (copy_to_user(buffer, &tu->queue[tu->qhead++], sizeof(snd_timer_read_t))) {
+			spin_unlock_irqrestore(&tu->qlock, flags);
+			return -EFAULT;
+		}
 		tu->qhead %= tu->queue_size;
 		tu->qused--;
-		snd_spin_unlock(tu, qlock, &flags);
+		spin_unlock_irqrestore(&tu->qlock, flags);
 		result += sizeof(snd_timer_read_t);
 		buffer += sizeof(snd_timer_read_t);
 	}
 	return result;
 }
 
-#ifdef SND_POLL
 static unsigned int snd_timer_user_poll(struct file *file, poll_table * wait)
 {
-        unsigned long flags;
         unsigned int mask;
         snd_timer_user_t *tu;
 
@@ -1125,10 +1129,7 @@ static unsigned int snd_timer_user_poll(struct file *file, poll_table * wait)
         if (!tu)
                 return -EIO;
 
-        snd_spin_lock(tu, qlock, &flags);
-        snd_getlock(tu, qchange) |= SND_WK_SLEEP;
-        snd_spin_unlock(tu, qlock, &flags);
-        snd_poll_wait(file, tu, qchange, wait);
+        poll_wait(file, &tu->qchange_sleep, wait);
 	
 	mask = 0;
 	if (tu->qused)
@@ -1136,38 +1137,6 @@ static unsigned int snd_timer_user_poll(struct file *file, poll_table * wait)
 
 	return mask;
 }
-#else
-static int snd_timer_user_select(struct file *file,
-				 int sel_type,
-				 select_table * wait)
-{
-        unsigned long flags;
-        snd_timer_user_t *tu;
-
-        tu = (snd_timer_user_t *) file->private_data;
-        if (!tu)
-                return -EIO;
-
-	switch (sel_type) {
-	case SEL_IN:
-		snd_spin_lock(tu, qlock, &flags);
-		if (!tu->qused) {
-			snd_getlock(tu, qchange) |= SND_WK_SLEEP;
-			snd_spin_unlock(tu, qchange, &flags);
-			snd_select_wait(tu, qchange, wait);
-			return 0;
-		}
-		snd_getlock(tu, qchange) &= ~SND_WK_SLEEP;
-		snd_spin_unlock(tu, qlock, &flags);
-		return 1;
-	case SEL_OUT:
-		break;
-	case SEL_EX:
-		break;
-	}
-	return 0;
-}
-#endif
 
 static snd_minor_t snd_timer_reg =
 {
@@ -1181,11 +1150,7 @@ static snd_minor_t snd_timer_reg =
 	NULL,			/* write */
 	snd_timer_user_open,	/* open */
 	snd_timer_user_release,	/* release */
-#ifdef SND_POLL
 	snd_timer_user_poll,	/* poll */
-#else
-	snd_timer_user_select,	/* select */
-#endif
 	snd_timer_user_ioctl,	/* ioctl */
 	NULL			/* mmap */
 };
@@ -1194,10 +1159,6 @@ static snd_minor_t snd_timer_reg =
  *  ENTRY functions
  */
 
-#ifndef LINUX_2_1
-extern struct symbol_table snd_symbol_table_timer_export;
-#endif
-
 static snd_info_entry_t *snd_timer_proc_entry = NULL;
 
 int init_module(void)
@@ -1205,10 +1166,6 @@ int init_module(void)
 	int err;
 	snd_info_entry_t *entry;
 
-#ifndef LINUX_2_1
-	if (register_symtab(&snd_symbol_table_timer_export) < 0)
-		return -ENOMEM;
-#endif
 #ifdef CONFIG_SND_OSSEMUL
 	snd_oss_info_register(SND_OSS_INFO_DEV_TIMERS, SND_CARDS - 1, "system timer");
 #endif
@@ -1245,3 +1202,18 @@ void cleanup_module(void)
 	snd_oss_info_unregister(SND_OSS_INFO_DEV_TIMERS, SND_CARDS - 1);
 #endif
 }
+
+EXPORT_SYMBOL(snd_timer_open);
+EXPORT_SYMBOL(snd_timer_open1);
+EXPORT_SYMBOL(snd_timer_open_slave);
+EXPORT_SYMBOL(snd_timer_close);
+EXPORT_SYMBOL(snd_timer_resolution);
+EXPORT_SYMBOL(snd_timer_start);
+EXPORT_SYMBOL(snd_timer_stop);
+EXPORT_SYMBOL(snd_timer_continue);
+EXPORT_SYMBOL(snd_timer_new_device);
+EXPORT_SYMBOL(snd_timer_free);
+EXPORT_SYMBOL(snd_timer_register);
+EXPORT_SYMBOL(snd_timer_unregister);
+EXPORT_SYMBOL(snd_timer_interrupt);
+EXPORT_SYMBOL(snd_timer_system_resolution);
