@@ -44,7 +44,7 @@ static struct {
 	{	0x83847604,	"SigmaTel STAC9701/3/4/5" },
 	{	0x83847605,	"SigmaTel STAC9704"	},
 	{	0x83847608,	"SigmaTel STAC9708"	},
-	{	0x83847609,	"SigmaTel STAC9721"	},
+	{	0x83847609,	"SigmaTel STAC9721/23"	},
 	{	0, 		NULL			}
 };
 
